package com.jpsoft.smart.modules.common.utils;

import com.jpsoft.smart.modules.wechat.entity.AccessToken;
import com.jpsoft.smart.modules.wechat.vo.UserInfo;
import net.sf.json.JSONException;
import net.sf.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.UnsupportedEncodingException;

/**
 * 公众平台通用接口工具类
 * 
 * @author lt
 * @date 2013-08-09
 */
public class WeixinUtil {
	private static Logger log = LoggerFactory.getLogger(WeixinUtil.class);
	
	// 获取access_token的接口地址（GET） 限200（次/天）
	public final static String access_token_url = "https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=APPID&secret=APPSECRET";

	//获取微信的code
	public static String code = "https://open.weixin.qq.com/connect/oauth2/authorize?appid=APPID&redirect_uri=REDIRECT_URI&response_type=code&scope=SCOPE&state=STATE#wechat_redirect";
	public static String getCodeRequest(String appid, String url,String scope){
        String result = null;
        String url_encode = "";
        try {
			url_encode = java.net.URLEncoder.encode(url, "utf-8");
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        result  = code.replace("APPID", appid).replace("REDIRECT_URI",url_encode).replace("SCOPE", scope);
        return result;
    }
	
	//通过code换取网页授权access_token
	public static String accessTokenurl = "https://api.weixin.qq.com/sns/oauth2/access_token?appid=APPID&secret=SECRET&code=CODE&grant_type=authorization_code";
	public static AccessToken getAccessToken(String appid, String appsecret,String code) throws Exception {
        AccessToken accessToken = null;
        String formatUrl = accessTokenurl.replace("APPID", appid).replace("SECRET",appsecret).replace("CODE", code);
        JSONObject json  = HttpConnectionUtil.httpRequest(formatUrl, "GET", null);
        if (!json.isNullObject() && !json.isEmpty()) {
			try {
				accessToken = new AccessToken();
				accessToken.setToken(json.getString("access_token"));
				accessToken.setExpiresIn(json.getInt("expires_in"));
				accessToken.setOpenid(json.getString("openid"));
				accessToken.setScope(json.getString("scope"));
			} catch (Exception e) {
				
				// 获取token失败
				log.error("获取token失败,返回结果result={}", json);
				if (json.getString("errcode").equals("40163")){
					throw new Exception("请重新扫码");

				}

			}
		}
        return accessToken;
    }
	
	public static String jsAccessTokenUrl = "https://api.weixin.qq.com/cgi-bin/ticket/getticket?access_token=ACCESS_TOKEN&type=jsapi";
	public static String getJsAPI(String accessToken){
		String formatUrl = jsAccessTokenUrl.replace("ACCESS_TOKEN", accessToken);
        JSONObject json  = HttpConnectionUtil.httpRequest(formatUrl, "GET", null);
        
        if (!json.isNullObject() && !json.isEmpty()) {
			try {
				return json.getString("ticket");
			} catch (Exception e) {
				// 获取token失败
				log.error("获取jsAPI失败,返回结果result={}", json);
			}
		}
        
        return null;
	}
	
	/**
	 * 获取access_token
	 * 
	 * @param appid 凭证
	 * @param appsecret 密钥
	 * @return
	 */
	public static AccessToken getAccessToken(String appid, String appsecret) {
		AccessToken accessToken = null;

		String requestUrl = access_token_url.replace("APPID", appid).replace("APPSECRET", appsecret);
		JSONObject jsonObject =HttpConnectionUtil.httpRequest(requestUrl, "GET", null);
		// 如果请求成功
		if (null != jsonObject) {
			try {
				accessToken = new AccessToken();
				accessToken.setToken(jsonObject.getString("access_token"));
				accessToken.setExpiresIn(jsonObject.getInt("expires_in"));
				//accessToken.setOpenid(jsonObject.getString("openid"));
			} catch (JSONException e) {
				accessToken = null;
				// 获取token失败
				log.error("获取token失败 errcode:{} errmsg:{}", jsonObject.getInt("errcode"), jsonObject.getString("errmsg"));
			}
		}
		return accessToken;
	}
	
	// 获取用户信息
	public static String user_info_url = "https://api.weixin.qq.com/sns/userinfo?access_token=ACCESS_TOKEN&openid=OPENID&lang=zh_CN";
	public static String subscribe_user_info_url = "https://api.weixin.qq.com/cgi-bin/user/info?access_token=ACCESS_TOKEN&openid=OPENID&lang=zh_CN ";
	
	public static UserInfo getUserInfo(String openid, String accessToken) {
		// 拼装url
		String url = user_info_url.replace("ACCESS_TOKEN", accessToken).replace("OPENID", openid);;
		
		JSONObject jsonObject = HttpConnectionUtil.httpRequest(url, "GET", null);
		
		UserInfo userInfo = null;
		 if (null != jsonObject) {
				try {
					userInfo = new UserInfo();
					userInfo.setOpenid(jsonObject.getString("openid").replaceAll("\"", ""));
					
					userInfo.setNickname(jsonObject.getString("nickname").replaceAll("\"", ""));
					userInfo.setCity(jsonObject.getString("city").replaceAll("\"", ""));
					userInfo.setProvince(jsonObject.getString("province").replaceAll("\"", ""));
					userInfo.setCountry(jsonObject.getString("country").replaceAll("\"", ""));
					userInfo.setHeadimgurl(jsonObject.getString("headimgurl").replaceAll("\"", ""));
				} catch (JSONException e) {
					// 获取token失败
					log.error("获取token失败 errcode:{} errmsg:{}", jsonObject.getInt("errcode"), jsonObject.getString("errmsg"));
				}
			}
		return userInfo;
	}
}
