package com.jpsoft.smart.modules.lapi.service;

import com.jpsoft.smart.modules.base.entity.DeviceInfo;
import com.jpsoft.smart.modules.lapi.vo.LapiMsgResult;
import com.jpsoft.smart.modules.lapi.vo.LapiResult;

import java.util.List;

/**
 * @author 墨鱼_mo
 * @date 2020-3-15 9:36
 */
public interface ILapiService {

    /**
     * 通过ip和port查看门禁设备，有设备信息返回为正常
     * @param ipAddress
     * @param port
     * @return
     * @throws Exception
     */
     DeviceInfo findByIpAddressAndPort(String ipAddress, String port) throws Exception;

    /**
     * 根据设备id查看设备在线状态
     * @param id 设备id
     * @return
     */
     Boolean keepAlive(String id) throws Exception;


    /**
     * 平台上传设备人脸信息
     * @param id
     * @return
     * @throws Exception
     */
     List<LapiMsgResult> addPerson(Long id) throws Exception;

    /**
     * 根据公司名称获取人脸库
     * @param companyName
     * @return
     */
     String getFaceDbId(DeviceInfo deviceInfo, String companyName) throws Exception;

    /**
     * 删除设备人脸数据
     * @param id
     * @return
     */
    List<LapiMsgResult> deletePerson(Long id) throws Exception;

    /**
     * 远程手机开门
     * @param deviceInfoId
     * @return
     * @throws Exception
     */
    Boolean phoneOpenDoor(String deviceInfoId) throws Exception;


    /**
     * 重启设备
     * @param deviceInfoId 设备id
     * @return
     */
    Boolean reBoot(String deviceInfoId) throws Exception;


    /**
     * 添加公司照片库
     * @param companyInfoId 公司id
     * @param deviceInfoIds 设备id
     * @return
     * @throws Exception
     */
    List<LapiResult> addCompanyLib(String companyInfoId, String deviceInfoIds) throws Exception;


    /**
     * 删除公司人脸库
     * @param deviceInfoId 设备id
     * @param libraryId 人脸库id
     * @return
     * @throws Exception
     */
    LapiResult deleteCompanyLib(String deviceInfoId,String libraryId) throws Exception;


    /**
     * 更新公司人脸库名称
     * @param companyId  公司id
     * @param deviceInfoId
     * @param libraryId
     * @return
     * @throws Exception
     */
    LapiResult updateCompanyLib(String companyId,String deviceInfoId,String libraryId) throws Exception;

    /**
     * 删除设备上的所有人脸库
     * @param deviceInfoId
     * @return
     * @throws Exception
     */
    List<LapiResult> deleteDeviceAllCompanyLib(String deviceInfoId) throws Exception;




}
