package com.jpsoft.smart.electricity.server.codec;

import com.jpsoft.smart.electricity.server.AttributeKeys;
import com.jpsoft.smart.electricity.server.protocol.MeterSendPacket;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.MessageToByteEncoder;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;

@Slf4j
public class MeterEncodeHandler extends MessageToByteEncoder<MeterSendPacket> {
    private final short START_SYMBOL = 0x58;
    private final short END_SYMBOL = 0x6B;

    @Override
    protected void encode(ChannelHandlerContext ctx, MeterSendPacket packet, ByteBuf out) throws Exception {
        short[] buffer;

        if (packet.getCommand()== MeterSendPacket.RECHARGE){
            buffer = rechargeBuild(packet.getMachineNo(),packet.getUnitNo(),
                    packet.getHouseNo(),
                    packet.getCommand(),
                    packet.getRechargeNum(),
                    packet.getRechargeTimes());
        }
        else{
            buffer = generalBuild(packet.getMachineNo(),packet.getUnitNo(),packet.getHouseNo(),packet.getCommand());
        }

        PrintRecvData(buffer);

        if(packet.getRequestTime()!=null){
            ctx.channel().attr(AttributeKeys.REQUEST_TIME).set(packet.getRequestTime());
        }

        for (int i=0;i<buffer.length;i++) {
            out.writeByte(buffer[i]);
        }
    }

    private void PrintRecvData(short[] buffer) {
        StringBuilder sb = new StringBuilder();

        for(int i=0;i<buffer.length;i++) {
            if (sb.length()>0){
                sb.append(" ");
            }

            short b = buffer[i];

            String str = Integer.toHexString(b);

            if (str.length()==1){
                sb.append("0");
            }

            sb.append(str.toUpperCase());
        }

        log.warn("发送数据:" + sb.toString());
    }

    public short[] generalBuild(int machineNo,int unitNo,int houseNo,int cmd)
    {
        ArrayList<Short> arr = new ArrayList<Short>();

        arr.add(START_SYMBOL);

        int bh1= machineNo & 0x00ff;
        int bh2= (machineNo & 0xff00) >> 8;

        arr.add((short)bh1);
        arr.add((short)bh2);

        int dw1= unitNo & 0x00ff;
        int dw2= (unitNo & 0xff00) >> 8;

        arr.add((short)dw1);
        arr.add((short)dw2);

        arr.add((short)houseNo);

        //读取命令
        arr.add((short)0x88);

        arr.add((short)cmd);

        int valCode1 = bh1 ^ bh2 ^ dw1 ^ dw2 ^ houseNo ^ cmd;

        arr.add((short)valCode1);

        arr.add((short)bh1);
        arr.add((short)bh2);
        arr.add((short)houseNo);

        for (int i = 0; i < 15; i++)
        {
            arr.add((short)0x33);
        }

        arr.add(END_SYMBOL);

        short[] buffer = toShortArray(arr);

        return buffer;
    }

    private short[] toShortArray(List<Short> arr) {
        short[] buffer = new short[arr.size()];

        for (int i = 0; i < arr.size(); i++) {
            buffer[i] = arr.get(i);
        }

        return buffer;
    }

    public short[] rechargeBuild(int machineNo, int unitNo, int houseNo,int cmd,int num,int times)
    {
        List<Short> arr = new ArrayList<Short>();

        arr.add(START_SYMBOL);

        int bh1= machineNo & 0x00ff;
        int bh2= (machineNo & 0xff00) >> 8;

        arr.add((short)bh1);
        arr.add((short)bh2);

        int dw1= unitNo & 0x00ff;
        int dw2= (unitNo & 0xff00) >> 8;

        arr.add((short)dw1);
        arr.add((short)dw2);

        arr.add((short)houseNo);

        arr.add((short)0x88);
        arr.add((short)cmd);

        short mb1=0x00;
        short mb2=0x00;

        arr.add(mb1);
        arr.add(mb2);

        Short sign = 0;
        int gdl = num;

        //售电命令： 命令字 0A 正电 A0 负电
        if(num>0){
            sign=(short)0x0A;
        }else{
            sign=(short)0xA0;
            gdl = -gdl;
        }

        arr.add(sign);

        int gdl1= (gdl & 0xff00) >> 8;
        int gdl2= gdl & 0x00ff;

        arr.add((short)gdl1);
        arr.add((short)gdl2);

        int gdcs1 = (times & 0xff00) >> 8;
        int gdcs2 = times & 0x00ff;

        arr.add((short)gdcs1);
        arr.add((short)gdcs2);

        //额定功率 6600W
        int gl = 6600;
        int xdgl1 = (gl & 0xff00) >> 8;
        int xdgl2 = gl & 0x00ff;

        arr.add((short)xdgl1);
        arr.add((short)xdgl2);

        //断电报警 20度
        int ddbj= 20;

        arr.add((short)ddbj);

        int valCode1 = (bh1 ^ bh2 ^ dw1 ^ dw2 ^ houseNo ^ cmd ^ times) + 0x98;
        valCode1 &= 0x00ff;

        arr.add((short)valCode1);

        int valCode2= (bh1+bh2+dw1+dw2+houseNo+cmd +times) ^ 0x68;
        valCode2 &= 0x00ff;

        arr.add((short)valCode2);

        int valCode3=(sign ^ gdl1 ^ gdl2 ^ cmd ^ times) + 0x78;
        valCode3 &= 0x00ff;

        arr.add((short)valCode3);

        int valCode4=(sign + gdl1 + gdl2 + cmd + times) ^ 0x76;
        valCode4 &= 0x00ff;

        arr.add((short)valCode4);

        int valCode5= (valCode1 ^ valCode2 ^ valCode3 ^ valCode4);
        valCode5 &= 0x00ff;

        arr.add((short)valCode5);

        //int sumCode=(bh1 ^ bh2 ^ dw1 ^ dw2 ^ houseNo ^ 0x88 ^ cmd ^ mb1 ^ mb2 ^ sign ^ gdl1 ^ gdl2 ^ gdcs1 ^ gdcs2 ^ xdgl1 ^ xdgl2 ^ ddbj ^ valCode1^ valCode2^ valCode3^ valCode4^ valCode5);
        int sumCode = bh1;

        for (int i = 2; i <= 22; i++)
        {
            sumCode ^= arr.get(i);
        }

        sumCode &= 0x00ff;

        arr.add((short)sumCode);

        arr.add((short)0x33);
        arr.add((short)0x33);
        arr.add((short)0x33);
        arr.add((short)0x6B);

        short[] buffer = toShortArray(arr);

        return buffer;
    }

}
