package com.jpsoft.smart.electricity.server.handler;

import com.jpsoft.smart.modules.base.service.TerminalInfoService;
import com.jpsoft.smart.electricity.server.AttributeKeys;
import com.jpsoft.smart.electricity.server.ChannelWrapper;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 管理channel
 */
@Slf4j
@Component
@ChannelHandler.Sharable
public class ActiveChannelHandler extends ChannelInboundHandlerAdapter {
    @Autowired
    private Map<String, ChannelWrapper> activeChannelMap;

    @Autowired
    private TerminalInfoService terminalInfoService;

    private ExecutorService threadPool = Executors.newFixedThreadPool(10);

    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");

        String uid = sdf.format(new Date());

        //我们也可以在 channelActive() 方法中，实现对客户端连接 ip 黑白名单的过滤
        log.warn("有新连接:" + uid + " " + ctx.channel().remoteAddress().toString());

        ctx.channel().attr(AttributeKeys.UID).set(uid);

        //activeChannelMap.put(uid,new ChannelWrapper(ctx.channel()));
        super.channelActive(ctx);
    }

    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        String uid = ctx.channel().attr(AttributeKeys.UID).get();
        String terminalNo = ctx.channel().attr(AttributeKeys.TERMINAL_NO).get();

        log.warn("连接已断开:" + uid + "," + terminalNo);

        if(terminalNo!=null){
            //修改终端为离线状态
            threadPool.submit(()-> {
                terminalInfoService.updateActiveByTerminalNo(terminalNo,false);
            });

            activeChannelMap.remove(terminalNo);
        }

        super.channelInactive(ctx);
    }
}
