package com.jpsoft.smart.electricity.server.handler;

import com.jpsoft.smart.modules.base.service.TerminalInfoService;
import com.jpsoft.smart.electricity.server.AttributeKeys;
import com.jpsoft.smart.electricity.server.ChannelWrapper;
import com.jpsoft.smart.electricity.server.protocol.HeartbeatReceivePacket;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Slf4j
@Component
@ChannelHandler.Sharable
public class HeartbeatServerHandler extends SimpleChannelInboundHandler<HeartbeatReceivePacket> {
    @Autowired
    private Map<String, ChannelWrapper> activeChannelMap;

    @Autowired
    private TerminalInfoService terminalInfoService;

    private ExecutorService threadPool = Executors.newFixedThreadPool(10);

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, HeartbeatReceivePacket packet) throws Exception {
        log.warn(ctx.channel().remoteAddress().toString() + "对应终端号为" + packet.getTerminalNo());

        /**
         * 只要有一个 channel 的一个 handler 中的 channelRead0() 方法阻塞了 NIO 线程，
         * 最终都会拖慢绑定在该 NIO 线程上的其他所有的 channel。
         * 对于耗时的操作，我们需要把这些耗时的操作丢到我们的业务线程池中去处理。
        **/
        if(ctx.channel().attr(AttributeKeys.TERMINAL_NO).get()==null){
            ctx.channel().attr(AttributeKeys.TERMINAL_NO).set(packet.getTerminalNo());
        }

        if(!activeChannelMap.containsKey(packet.getTerminalNo())) {
            activeChannelMap.put(packet.getTerminalNo(), new ChannelWrapper(ctx.channel()));
            log.warn(packet.getTerminalNo() + "已加入到activeChannelMap");

            //设置终端为在线状态
            threadPool.submit(()-> {
                terminalInfoService.updateActiveByTerminalNo(packet.getTerminalNo(),true);
            });
        }
    }
}
