package com.jpsoft.smart.electricity.server.handler;

import com.jpsoft.smart.electricity.server.AttributeKeys;
import com.jpsoft.smart.electricity.server.ChannelWrapper;
import com.jpsoft.smart.electricity.server.protocol.MeterReceivePacket;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Map;
import java.util.concurrent.BlockingQueue;

@Slf4j
@Component
@ChannelHandler.Sharable
public class MeterServerHandler extends SimpleChannelInboundHandler<MeterReceivePacket> {
    @Autowired
    private Map<String, ChannelWrapper> activeChannelMap;

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, MeterReceivePacket msg) throws Exception {
        log.warn("收到电表数据：" + msg.toString());

        //终端编号
        String terminalNo = ctx.channel().attr(AttributeKeys.TERMINAL_NO).get();

        msg.setTerminalNo(terminalNo);

        //表号+单元号+户号
        String key = Integer.toHexString(msg.getMachineNo()).toUpperCase() + "-" + Integer.toHexString(msg.getUnitNo()).toUpperCase() + "-" + msg.getHouseNo();

        //放入队列
        BlockingQueue<MeterReceivePacket> queue = activeChannelMap.get(terminalNo).getReceivePacketQueue(key);

        queue.put(msg);
    }
}
