package com.jpsoft.smart.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.smart.modules.base.dto.CompanyInfoDTO;
import com.jpsoft.smart.modules.base.entity.CompanyPosition;
import com.jpsoft.smart.modules.base.service.CompanyPositionService;
import com.jpsoft.smart.modules.common.utils.PojoUtils;
import com.jpsoft.smart.modules.common.dto.Sort;
import com.jpsoft.smart.modules.common.dto.MessageResult;
import com.jpsoft.smart.modules.base.entity.CompanyInfo;
import com.jpsoft.smart.modules.base.service.CompanyInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/companyInfo")
@Api(description = "companyInfo")
public class CompanyInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private CompanyInfoService companyInfoService;

    @Autowired
    private CompanyPositionService companyPositionService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<CompanyInfoDTO> create(){
        MessageResult<CompanyInfoDTO> msgResult = new MessageResult<>();

        CompanyInfoDTO companyInfoDTO = new CompanyInfoDTO();

        msgResult.setData(companyInfoDTO);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<Map> add(@RequestBody CompanyInfoDTO companyInfoDTO,@RequestAttribute String subject){
        MessageResult<Map> msgResult = new MessageResult<>();
        Map<String,Object> map = new HashMap<>();

        try {
            CompanyInfo companyInfo = new CompanyInfo();
            PojoUtils.map(companyInfoDTO, companyInfo);
            companyInfo.setId(UUID.randomUUID().toString());
            companyInfo.setDelFlag(false);
            companyInfo.setCreateBy(subject);
            companyInfo.setCreateTime(new Date());
            companyInfoService.insert(companyInfo);
            map.put("companyInfo",companyInfo);

            CompanyPosition companyPosition = new CompanyPosition();
            companyPosition.setId(UUID.randomUUID().toString());
            companyPosition.setCompanyId(companyInfo.getId());
            companyPosition.setPosition1Name(companyInfoDTO.getPosition1Name());
            companyPosition.setPosition2Name(companyInfoDTO.getPosition2Name());
            companyPosition.setPosition3Name(companyInfoDTO.getPosition3Name());
            companyPosition.setPosition4Name(companyInfoDTO.getPosition4Name());
            companyPosition.setPosition5Name(companyInfoDTO.getPosition5Name());
            companyPosition.setDelFlag(false);
            companyPosition.setCreateBy(subject);
            companyPosition.setCreateTime(new Date());
            int affectCount = companyPositionService.insert(companyPosition);
            map.put("companyPosition",companyPosition);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(map);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<CompanyInfoDTO> edit(@PathVariable("id") String id){
        MessageResult<CompanyInfoDTO> msgResult = new MessageResult<>();

        try {
            CompanyInfo companyInfo = companyInfoService.get(id);
            CompanyInfoDTO companyInfoDTO = new CompanyInfoDTO();

            if (companyInfo != null) {
                PojoUtils.map(companyInfo, companyInfoDTO);

                CompanyPosition companyPosition = new CompanyPosition();
                Map<String,Object> searchParams1 = new HashMap<>();
                searchParams1.put("companyId",companyInfo.getId());
                List<Sort> sortList1 = new ArrayList<>();
                sortList1.add(new Sort("id_","asc"));
                Page<CompanyPosition> page1 = companyPositionService.pageSearch(searchParams1,1,1,false,sortList1);
                if(page1.size() > 0) {
                    companyPosition = page1.get(0);
                    companyInfoDTO.setPosition1Name(companyPosition.getPosition1Name());
                    companyInfoDTO.setPosition2Name(companyPosition.getPosition2Name());
                    companyInfoDTO.setPosition3Name(companyPosition.getPosition3Name());
                    companyInfoDTO.setPosition4Name(companyPosition.getPosition4Name());
                    companyInfoDTO.setPosition5Name(companyPosition.getPosition5Name());
                }

                msgResult.setResult(true);
                msgResult.setData(companyInfoDTO);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新")
    @PostMapping("update")
    public MessageResult<Map> update(@RequestBody CompanyInfoDTO companyInfoDTO,@RequestAttribute String subject){
        MessageResult<Map> msgResult = new MessageResult<>();
        Map<String,Object> map = new HashMap<>();

        try {
            int affectCount = 0;
            CompanyInfo companyInfo = new CompanyInfo();
            PojoUtils.map(companyInfoDTO, companyInfo);
            companyInfo.setUpdateBy(subject);
            companyInfo.setUpdateTime(new Date());
            companyInfoService.update(companyInfo);
            map.put("companyInfo",companyInfo);
            affectCount++;

            CompanyPosition companyPosition = new CompanyPosition();
            Map<String,Object> searchParams1 = new HashMap<>();
            searchParams1.put("companyId",companyInfo.getId());
            List<Sort> sortList1 = new ArrayList<>();
            sortList1.add(new Sort("id_","asc"));
            Page<CompanyPosition> page1 = companyPositionService.pageSearch(searchParams1,1,1,false,sortList1);
            if(page1.size() > 0) {
                companyPosition = page1.get(0);
                companyPosition.setPosition1Name(companyInfoDTO.getPosition1Name());
                companyPosition.setPosition2Name(companyInfoDTO.getPosition2Name());
                companyPosition.setPosition3Name(companyInfoDTO.getPosition3Name());
                companyPosition.setPosition4Name(companyInfoDTO.getPosition4Name());
                companyPosition.setPosition5Name(companyInfoDTO.getPosition5Name());
                companyPosition.setUpdateBy(subject);
                companyPosition.setUpdateTime(new Date());
                companyPositionService.update(companyPosition);
                affectCount++;
            }
            map.put("companyPosition",companyPosition);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(map);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            CompanyInfo companyInfo = companyInfoService.get(id);
            companyInfo.setDelFlag(true);
            companyInfo.setUpdateBy(subject);
            companyInfo.setUpdateTime(new Date());

            int affectCount = companyInfoService.update(companyInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                CompanyInfo companyInfo = companyInfoService.get(id);
                companyInfo.setDelFlag(true);
                companyInfo.setUpdateBy(subject);
                companyInfo.setUpdateTime(new Date());

                affectCount += companyInfoService.update(companyInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("sort_no","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<CompanyInfo> page = companyInfoService.pageSearch(searchParams,pageIndex,pageSize,sortList);
        Page<CompanyInfoDTO> pageDTO = PojoUtils.convertPage(page, CompanyInfoDTO.class);
        for(CompanyInfoDTO li : pageDTO.getResult()){
            Map<String,Object> searchParams1 = new HashMap<>();
            searchParams1.put("companyId",li.getId());
            List<Sort> sortList1 = new ArrayList<>();
            sortList1.add(new Sort("id_","asc"));

            Page<CompanyPosition> page1 = companyPositionService.pageSearch(searchParams1,1,1,false,sortList1);
            if(page1.size() > 0) {
                li.setPosition1Name(page1.get(0).getPosition1Name());
                li.setPosition2Name(page1.get(0).getPosition2Name());
                li.setPosition3Name(page1.get(0).getPosition3Name());
                li.setPosition4Name(page1.get(0).getPosition4Name());
                li.setPosition5Name(page1.get(0).getPosition5Name());
            }
        }

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(pageDTO));

        return msgResult;
    }

    @ApiOperation(value="所有单位列表")
    @RequestMapping(value = "list",method = RequestMethod.POST)
    public MessageResult<List<CompanyInfo>> list(){
        MessageResult<List<CompanyInfo>> msgResult = new MessageResult<>();

        List<CompanyInfo> list = companyInfoService.list();

        msgResult.setResult(true);
        msgResult.setData(list);

        return msgResult;
    }
}
