package com.jpsoft.smart.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.smart.modules.base.entity.AlarmInfo;
import com.jpsoft.smart.modules.common.utils.PojoUtils;
import com.jpsoft.smart.modules.common.dto.Sort;
import com.jpsoft.smart.modules.common.dto.MessageResult;
import com.jpsoft.smart.modules.base.entity.DeviceInfo;
import com.jpsoft.smart.modules.base.service.DeviceInfoService;
import com.jpsoft.smart.modules.lapi.service.ILapiService;
import com.jpsoft.smart.modules.sys.service.DataDictionaryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/deviceInfo")
@Api(description = "设备信息")
public class DeviceInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private DeviceInfoService deviceInfoService;

    @Autowired
    private DataDictionaryService dataDictionaryService;

    @Autowired
    private ILapiService lapiService;

    @ApiOperation(value="添加设备")
    @PostMapping("add")
    public MessageResult<DeviceInfo> add(@RequestBody DeviceInfo deviceInfo,@RequestAttribute String subject){
        MessageResult<DeviceInfo> msgResult = new MessageResult<>();

        try {
            DeviceInfo item = deviceInfoService.getByDeviceNo(deviceInfo.getDeviceNo());

            if(item == null) {

                deviceInfo.setId(UUID.randomUUID().toString());
                deviceInfo.setDelFlag(false);
                deviceInfo.setCreateBy(subject);
                deviceInfo.setCreateTime(new Date());

                int affectCount = deviceInfoService.insert(deviceInfo);

                if (affectCount > 0) {
                    msgResult.setResult(true);
                    msgResult.setData(deviceInfo);
                } else {
                    msgResult.setResult(false);
                    msgResult.setMessage("数据库添加失败");
                }
            }
            else{
                msgResult.setResult(false);
                msgResult.setMessage("设备编号已经存在，不能重复添加！");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<DeviceInfo> create(){
        MessageResult<DeviceInfo> msgResult = new MessageResult<>();

        DeviceInfo deviceInfo = new DeviceInfo();

        msgResult.setData(deviceInfo);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value="获取设备信息")
    @GetMapping("edit/{id}")
    public MessageResult<DeviceInfo> edit(@PathVariable("id") String id){
        MessageResult<DeviceInfo> msgResult = new MessageResult<>();

        try {
            DeviceInfo deviceInfo = deviceInfoService.get(id);

            if (deviceInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(deviceInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新设备信息")
    @PostMapping("update")
    public MessageResult<DeviceInfo> update(@RequestBody DeviceInfo deviceInfo,@RequestAttribute String subject){
        MessageResult<DeviceInfo> msgResult = new MessageResult<>();

        try {
            DeviceInfo item = deviceInfoService.getByDeviceNo(deviceInfo.getDeviceNo());

            if(item!=null) {
                //如果修改自己的属性
                if (item.getId().equals(deviceInfo.getId())) {
                    deviceInfo.setUpdateBy(subject);
                    deviceInfo.setUpdateTime(new Date());

                    int affectCount = deviceInfoService.update(deviceInfo);

                    if (affectCount > 0) {
                        msgResult.setResult(true);
                        msgResult.setData(deviceInfo);
                    } else {
                        msgResult.setResult(false);
                        msgResult.setMessage("数据库更新失败");
                    }
                } else {
                    //如果修改设备编号在数据库中已有重复的设备号，则不能修改！
                    msgResult.setResult(false);
                    msgResult.setMessage("该设备编号的设备已存在，不能修改！");
                }
            }
            else{
                msgResult.setResult(false);
                msgResult.setMessage("该设备不存在！");
            }

        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除设备")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            DeviceInfo deviceInfo = deviceInfoService.get(id);
            deviceInfo.setDelFlag(true);
            deviceInfo.setUpdateBy(subject);
            deviceInfo.setUpdateTime(new Date());

            int affectCount = deviceInfoService.update(deviceInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除设备")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                DeviceInfo deviceInfo = deviceInfoService.get(id);
                deviceInfo.setDelFlag(true);
                deviceInfo.setUpdateBy(subject);
                deviceInfo.setUpdateTime(new Date());

                affectCount += deviceInfoService.update(deviceInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="设备列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "aliasName",value = "设备别名",  paramType = "query"),
            @ApiImplicitParam(name = "deviceNo",value = "编号",  paramType = "query"),
            @ApiImplicitParam(name = "address",value = "位置",  paramType = "query"),
            @ApiImplicitParam(name = "companyId",value = "企业ID",  paramType = "query")
    })
    public MessageResult<Map> pageList(
            String aliasName,String deviceNo,String address,String companyId,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("a.create_time","desc"));

        if (StringUtils.isNotEmpty(aliasName)) {
            searchParams.put("aliasName","%" + aliasName + "%");
        }
        if (StringUtils.isNotEmpty(deviceNo)) {
            searchParams.put("deviceNo","%" + deviceNo + "%");
        }
        if (StringUtils.isNotEmpty(address)) {
            searchParams.put("address","%" + address + "%");
        }
        if (StringUtils.isNotEmpty(companyId)) {
            searchParams.put("companyId",companyId);
        }
        Page<DeviceInfo> page = deviceInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        for (DeviceInfo deviceInfo:page) {
            String typeN = dataDictionaryService.findParentId("ddf2ecbe-84a6-4f0d-a84f-2fc6917f5856",deviceInfo.getType());
            deviceInfo.setTypeN(typeN);
        }

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }


    @ApiOperation(value="根据IP地址和端口号查找设备")
    @PostMapping("getByIpAddressAndPort")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ipAddress",value = "IP地址", paramType = "query"),
            @ApiImplicitParam(name = "port",value = "端口号", paramType = "query")
    })
    public MessageResult<DeviceInfo> getByIpAddressAndPort( String ipAddress,String port,@RequestAttribute String subject) {
        MessageResult<DeviceInfo> msgResult = new MessageResult<>();
        try {
            DeviceInfo device = lapiService.findByIpAddressAndPort(ipAddress, port);

            msgResult.setResult(true);
            msgResult.setData(device);
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }
        return msgResult;
    }


    @ApiOperation(value="获取设备信息列表")
    @PostMapping("list")
    public MessageResult<List<DeviceInfo>> list(@RequestAttribute String subject) {
        MessageResult<List<DeviceInfo>> msgResult = new MessageResult<>();
        try {
            List<DeviceInfo> list = deviceInfoService.list();

            for (DeviceInfo deviceInfo:list) {
                String typeN = dataDictionaryService.findParentId("ddf2ecbe-84a6-4f0d-a84f-2fc6917f5856",deviceInfo.getType());
                deviceInfo.setTypeN(typeN);
            }

            msgResult.setResult(true);
            msgResult.setData(list);
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }
        return msgResult;
    }
}
