package com.jpsoft.smart.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.smart.modules.base.entity.ElectricClientInfo;
import com.jpsoft.smart.modules.base.entity.RoomInfo;
import com.jpsoft.smart.modules.base.service.ElectricClientInfoService;
import com.jpsoft.smart.modules.base.service.RoomInfoService;
import com.jpsoft.smart.modules.business.service.RechargeService;
import com.jpsoft.smart.modules.common.dto.MessageResult;
import com.jpsoft.smart.modules.common.dto.Sort;
import com.jpsoft.smart.electricity.server.protocol.MeterReceivePacket;
import com.jpsoft.smart.electricity.server.protocol.MeterSendPacket;
import com.jpsoft.smart.modules.sys.service.DataDictionaryService;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.concurrent.TimeUnit;

@RestController
@RequestMapping("/base/electricClientInfo")
public class ElectricClientInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private ElectricClientInfoService electricClientInfoService;

    @Autowired
    private RoomInfoService roomInfoService;

    @Autowired
    private DataDictionaryService dataDictionaryService;

    @Autowired
    private RechargeService rechargeService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<ElectricClientInfo> create(){
        MessageResult<ElectricClientInfo> msgResult = new MessageResult<>();

        ElectricClientInfo electricClientInfo = new ElectricClientInfo();

        msgResult.setData(electricClientInfo);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<ElectricClientInfo> add(@RequestBody ElectricClientInfo electricClientInfo,@RequestAttribute String subject){
        MessageResult<ElectricClientInfo> msgResult = new MessageResult<>();

        try {
            electricClientInfo.setId(UUID.randomUUID().toString());
            electricClientInfo.setDelFlag(false);
            electricClientInfo.setCreateBy(subject);
            electricClientInfo.setCreateTime(new Date());

            //RoomInfo roomInfo = roomInfoService.get(electricClientInfo.getRoomId());
            //金额计算
            //获取房间用电类型
//            String uType = "";
//            BigDecimal ePrice ;
//            BigDecimal erPrice = new BigDecimal(electricClientInfo.getElectricityRemaining());
//            if(StringUtils.isNotEmpty(roomInfo.getUseType())) {
//                ePrice =  new BigDecimal(dataDictionaryService.getValue(roomInfo.getUseType()));
//                BigDecimal totalPrice = ePrice.multiply(erPrice);
//                electricClientInfo.setAmountRemaining(totalPrice);
//            }

            electricClientInfo.setActive(false);

            int haveCount = electricClientInfoService.getCountByCNo(electricClientInfo.getCustomerNo(),electricClientInfo.getId());
            if(haveCount > 0){
                throw new Exception("该户号已存在!");
            }

            //查询该户当前电量
            MeterReceivePacket mrp = rechargeService.sendCommand(electricClientInfo.getRoomId(), MeterSendPacket.READ,3, TimeUnit.SECONDS);

            electricClientInfo.setRechargeTimes(mrp.getRechargeTimes());
            electricClientInfo.setElectricityRemaining(mrp.getRemain());
            int affectCount = electricClientInfoService.insert(electricClientInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(electricClientInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<ElectricClientInfo> edit(@PathVariable("id") String id){
        MessageResult<ElectricClientInfo> msgResult = new MessageResult<>();

        try {
            ElectricClientInfo electricClientInfo = electricClientInfoService.get(id);

            if (electricClientInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(electricClientInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="同步设备信息")
    @GetMapping("syncDevice")
    public MessageResult<ElectricClientInfo> syncDevice(String id){
        MessageResult<ElectricClientInfo> msgResult = new MessageResult<>();

        try {
            ElectricClientInfo electricClientInfo = electricClientInfoService.get(id);

            //查询该户当前电量
            MeterReceivePacket mrp = rechargeService.sendCommand(electricClientInfo.getRoomId(), MeterSendPacket.READ,3, TimeUnit.SECONDS);
            int affectCount = 0;

            if(mrp!=null){
                electricClientInfo.setRechargeTimes(mrp.getRechargeTimes());
                electricClientInfo.setElectricityRemaining(mrp.getRemain());
                electricClientInfo.setUpdateTime(new Date());

                affectCount = electricClientInfoService.update(electricClientInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(electricClientInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("设备同步失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<ElectricClientInfo> update(@RequestBody ElectricClientInfo electricClientInfo,@RequestAttribute String subject){
        MessageResult<ElectricClientInfo> msgResult = new MessageResult<>();

        try {
            electricClientInfo.setUpdateBy(subject);
            electricClientInfo.setUpdateTime(new Date());

            //RoomInfo roomInfo = roomInfoService.get(electricClientInfo.getRoomId());
            //金额计算
            //获取房间用电类型
//            String uType = "";
//            BigDecimal ePrice ;
//            BigDecimal erPrice = new BigDecimal(electricClientInfo.getElectricityRemaining());
//            if(StringUtils.isNotEmpty(roomInfo.getUseType())) {
//                ePrice =  new BigDecimal(dataDictionaryService.getValue(roomInfo.getUseType()));
//                BigDecimal totalPrice = ePrice.multiply(erPrice);
//                electricClientInfo.setAmountRemaining(totalPrice);
//            }

            int haveCount = electricClientInfoService.getCountByCNo(electricClientInfo.getCustomerNo(),electricClientInfo.getId());
            if(haveCount > 0){
                throw new Exception("该户号已存在!");
            }

            int affectCount = electricClientInfoService.update(electricClientInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(electricClientInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            ElectricClientInfo electricClientInfo = electricClientInfoService.get(id);
            electricClientInfo.setDelFlag(true);
            electricClientInfo.setUpdateBy(subject);
            electricClientInfo.setUpdateTime(new Date());

            int affectCount = electricClientInfoService.update(electricClientInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                ElectricClientInfo electricClientInfo = electricClientInfoService.get(id);
                electricClientInfo.setDelFlag(true);
                electricClientInfo.setUpdateBy(subject);
                electricClientInfo.setUpdateTime(new Date());

                affectCount += electricClientInfoService.update(electricClientInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(name="customerNo",defaultValue = "") String customerNo,
            @RequestParam(name="meterName",defaultValue = "") String meterName,
            @RequestParam(name = "areaId", defaultValue = "") String areaId,
            @RequestParam(name = "buildingId", defaultValue = "") String buildingId,
            @RequestParam(name = "roomsId", defaultValue = "") String roomsId,
            @RequestParam(name="pageIndex",defaultValue = "1") int pageIndex,
            @RequestParam(name="pageSize",defaultValue = "10") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }
        if (StringUtils.isNotEmpty(customerNo)) {
            searchParams.put("customerNo",customerNo);
        }
        if (StringUtils.isNotEmpty(meterName)) {
            searchParams.put("meterName","%" + meterName + "%");
        }

        //先判断是否选择房间
        if (StringUtils.isNotEmpty(roomsId)) {
            //选择了房间
            List<String> roomsIds = new ArrayList();
            roomsIds.add(roomsId);

            searchParams.put("roomsIds", roomsIds);
        }else if (StringUtils.isNotEmpty(buildingId)) {
            //没有选择房间,选择楼栋
            List<String> buildingIds = new ArrayList();
            buildingIds.add(buildingId);
            List<String> roomsIds = roomInfoService.findIdByTypeAndParentIds("3",buildingIds);
            if(roomsIds.size() < 1){
                roomsIds.add("");
            }
            searchParams.put("roomsIds", roomsIds);
        }else if (StringUtils.isNotEmpty(areaId)) {
            //没有选择房间和楼栋,选择的园区
            List<String> areaIds = new ArrayList();
            areaIds.add(areaId);
            List<String> buildingIds = roomInfoService.findIdByTypeAndParentIds("2",areaIds);
            List<String> roomsIds = roomInfoService.findIdByTypeAndParentIds("3",buildingIds);
            if(roomsIds.size() < 1){
                roomsIds.add("");
            }
            searchParams.put("roomsIds", roomsIds);
        }

        Page<ElectricClientInfo> page = electricClientInfoService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        List<ElectricClientInfo> resultClientList = new ArrayList<>();
        List<ElectricClientInfo> clientList = page.getResult();
        for(ElectricClientInfo clientInfo :clientList){
            RoomInfo room = roomInfoService.get(clientInfo.getRoomId());
            if(room == null){
                resultClientList.add(clientInfo);
                continue;
            }
            RoomInfo building = roomInfoService.get(room.getParentId());
            if(building == null){
                clientInfo.setRoom(room);
                resultClientList.add(clientInfo);
                continue;
            }
            RoomInfo area = roomInfoService.get(building.getParentId());
            if(area == null){
                clientInfo.setRoom(room);
                clientInfo.setBuilding(building);
                resultClientList.add(clientInfo);
                continue;
            }

            clientInfo.setRoom(room);
            clientInfo.setBuilding(building);
            clientInfo.setArea(area);
            resultClientList.add(clientInfo);
        }

        Map<String,Object> pageMap = new HashMap<>();
        pageMap.put("recordsTotal",page.getTotal());
        pageMap.put("recordsFiltered",page.getTotal());
        pageMap.put("totalPage",page.getPages());
        pageMap.put("pageNumber",page.getPageNum());
        pageMap.put("pageSize",page.getPageSize());
        pageMap.put("data", resultClientList);

        msgResult.setResult(true);
        msgResult.setData(pageMap);
        return msgResult;
    }

    @ApiOperation(value="打开或关闭电源")
    @PostMapping("powerSwitch")
    public MessageResult<MeterReceivePacket> powerSwitch(String roomId, int cmd) {
        MessageResult<MeterReceivePacket> msgResult = new MessageResult<>();

        try {
            MeterReceivePacket mrp = null;

            if(cmd==1) {
                mrp = rechargeService.sendCommand(roomId, MeterSendPacket.ON,5, TimeUnit.SECONDS);
            }
            else {
                mrp = rechargeService.sendCommand(roomId, MeterSendPacket.OFF,5,TimeUnit.SECONDS);
            }

            if(mrp!=null && mrp.isValidateResult()){
                ElectricClientInfo clientInfo = electricClientInfoService.findOneByRoomId(roomId);

                clientInfo.setActive(mrp.getState()!=1);

                electricClientInfoService.update(clientInfo);

                msgResult.setResult(true);
                msgResult.setData(mrp);
            }
            else{
                msgResult.setResult(false);
                msgResult.setMessage("命令超时!");
            }

        } catch (Exception e) {
            logger.error(e.getMessage(),e);

            msgResult.setMessage(e.getMessage());
            msgResult.setResult(false);
        }

        return msgResult;
    }
}
