package com.jpsoft.smart.modules.base.controller;

import com.github.pagehelper.Page;
import com.github.pagehelper.util.StringUtil;
import com.jpsoft.smart.modules.base.entity.DeviceInfo;
import com.jpsoft.smart.modules.base.service.DeviceInfoService;
import com.jpsoft.smart.modules.common.utils.PojoUtils;
import com.jpsoft.smart.modules.common.dto.Sort;
import com.jpsoft.smart.modules.common.dto.MessageResult;
import com.jpsoft.smart.modules.sys.entity.User;
import com.jpsoft.smart.modules.base.entity.PersonDeviceRelation;
import com.jpsoft.smart.modules.base.service.PersonDeviceRelationService;
import com.jpsoft.smart.modules.sys.service.DataDictionaryService;
import com.jpsoft.smart.modules.sys.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/personDeviceRelation")
@Api(description = "设备和人员绑定")
public class PersonDeviceRelationController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private PersonDeviceRelationService personDeviceRelationService;

    @Autowired
    private DeviceInfoService deviceInfoService;

    @Autowired
    private UserService userService;

    @Autowired
    private DataDictionaryService dataDictionaryService;

    
    @ApiOperation(value="添加设备和人员绑定")
    @PostMapping("add")
    public MessageResult<PersonDeviceRelation> add(@RequestBody PersonDeviceRelation personDeviceRelation,@RequestAttribute String subject){
        MessageResult<PersonDeviceRelation> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            PersonDeviceRelation item = personDeviceRelationService.findByDeviceIdAndPersonId(personDeviceRelation.getDeviceId(),personDeviceRelation.getPersonId());

            if(item == null) {

                personDeviceRelation.setId(UUID.randomUUID().toString());
                personDeviceRelation.setDelFlag(false);
                personDeviceRelation.setCreateBy(subject);
                personDeviceRelation.setCreateTime(new Date());

                affectCount = personDeviceRelationService.insert(personDeviceRelation);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(personDeviceRelation);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="人员与设备批量绑定")
    @PostMapping("batchBindDevice")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "personId",value = "人员ID", required = true,paramType = "query"),
            @ApiImplicitParam(name = "devices",value = "设备ID（多个设备）", required = true, paramType = "query")
    })
    public MessageResult<Integer> batchBindDevice(Long personId,String devices,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();
        int affectCount = 0;

        try {
            if(StringUtil.isNotEmpty(devices)){
               String[] deviceArray = devices.split(",");

               for (String deviceId:deviceArray) {

                   PersonDeviceRelation item = personDeviceRelationService.findByDeviceIdAndPersonId(deviceId,personId);

                   if(item == null) {
                       item = new PersonDeviceRelation();
                       item.setId(UUID.randomUUID().toString());
                       item.setDeviceId(deviceId);
                       item.setPersonId(personId);
                       item.setDelFlag(false);
                       item.setCreateBy(subject);
                       item.setCreateTime(new Date());

                       affectCount += personDeviceRelationService.insert(item);
                   }
               }
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("人员与0个设备绑定成功！");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }



    @ApiOperation(value="多个人员与设备批量绑定")
    @PostMapping("batchBindPerson")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "persons",value = "人员ID(多个人员)", required = true,paramType = "query"),
            @ApiImplicitParam(name = "deviceId",value = "设备ID", required = true, paramType = "query")
    })
    public MessageResult<Integer> batchBindPerson(String persons,String deviceId,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();
        int affectCount = 0;

        try {
            if(StringUtil.isNotEmpty(persons)){

                String[] personArray = persons.split(",");

                for (String strPersonId:personArray) {

                    long personId = Long.parseLong(strPersonId);

                    PersonDeviceRelation item = personDeviceRelationService.findByDeviceIdAndPersonId(deviceId,personId);

                    if(item == null) {
                        item = new PersonDeviceRelation();
                        item.setId(UUID.randomUUID().toString());
                        item.setDeviceId(deviceId);
                        item.setPersonId(personId);
                        item.setDelFlag(false);
                        item.setCreateBy(subject);
                        item.setCreateTime(new Date());

                        affectCount += personDeviceRelationService.insert(item);
                    }
                }
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("有0个人员与设备绑定成功！");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取人员和设备绑定信息")
    @GetMapping("edit/{id}")
    public MessageResult<PersonDeviceRelation> edit(@PathVariable("id") String id){
        MessageResult<PersonDeviceRelation> msgResult = new MessageResult<>();

        try {
            PersonDeviceRelation personDeviceRelation = personDeviceRelationService.get(id);

            if (personDeviceRelation != null) {
                msgResult.setResult(true);
                msgResult.setData(personDeviceRelation);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新设备和人员绑定")
    @PostMapping("update")
    public MessageResult<PersonDeviceRelation> update(@RequestBody PersonDeviceRelation personDeviceRelation,@RequestAttribute String subject){
        MessageResult<PersonDeviceRelation> msgResult = new MessageResult<>();

        try {
            personDeviceRelation.setUpdateBy(subject);
            personDeviceRelation.setUpdateTime(new Date());
            
            int affectCount = personDeviceRelationService.update(personDeviceRelation);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(personDeviceRelation);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除设备和人员绑定")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            PersonDeviceRelation personDeviceRelation = personDeviceRelationService.get(id);
            personDeviceRelation.setDelFlag(true);
            personDeviceRelation.setUpdateBy(subject);
            personDeviceRelation.setUpdateTime(new Date());

            int affectCount = personDeviceRelationService.update(personDeviceRelation);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                PersonDeviceRelation personDeviceRelation = personDeviceRelationService.get(id);
                personDeviceRelation.setDelFlag(true);
                personDeviceRelation.setUpdateBy(subject);
                personDeviceRelation.setUpdateTime(new Date());

                affectCount += personDeviceRelationService.update(personDeviceRelation);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "deviceId",value = "设备ID",  paramType = "query"),
            @ApiImplicitParam(name = "personId",value = "人员ID",  paramType = "query")
    })
    public MessageResult<Map> pageList(
            String deviceId,String personId,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (StringUtils.isNotEmpty(deviceId)) {
            searchParams.put("deviceId",deviceId);
        }

        if (StringUtils.isNotEmpty(personId)) {
            searchParams.put("personId",personId);
        }

        Page<PersonDeviceRelation> page = personDeviceRelationService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }


    @ApiOperation(value="根据人员ID查询设备列表")
    @RequestMapping(value = "listByPersonId",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "personId",value = "人员ID",paramType = "query")
    })
    public MessageResult<List<PersonDeviceRelation>> listByPersonId(Long personId){

        MessageResult<List<PersonDeviceRelation>> msgResult = new MessageResult<>();

        List<PersonDeviceRelation> personDeviceRelationList = personDeviceRelationService.findByPersonId(personId);

        msgResult.setResult(true);
        msgResult.setData(personDeviceRelationList);

        return msgResult;
    }



    @ApiOperation(value="根据设备ID查询人员列表")
    @RequestMapping(value = "listByDeviceId",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "deviceId",value = "设备ID",paramType = "query")
    })
    public MessageResult<List<PersonDeviceRelation>> listByDeviceId(String deviceId){

        MessageResult<List<PersonDeviceRelation>> msgResult = new MessageResult<>();

        List<PersonDeviceRelation> personDeviceRelationList = personDeviceRelationService.findByDeviceId(deviceId);

        msgResult.setResult(true);
        msgResult.setData(personDeviceRelationList);

        return msgResult;
    }


    @ApiOperation(value="人员解绑所有设备")
    @RequestMapping(value = "unbindDevice",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "personId",value = "人员ID",paramType = "query")
    })
    public MessageResult<Integer> unbindDevice(Long personId,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try{
            List<PersonDeviceRelation> personDeviceRelationList = personDeviceRelationService.findByPersonId(personId);

            int affectCount = 0;

            for (PersonDeviceRelation personDeviceRelation: personDeviceRelationList){
                personDeviceRelation.setDelFlag(true);
                personDeviceRelation.setUpdateBy(subject);
                personDeviceRelation.setUpdateTime(new Date());
                affectCount += personDeviceRelationService.update(personDeviceRelation);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="人员查询是否绑定设备")
    @RequestMapping(value = "isUnbindDeviceList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "personId",value = "人员ID",required = true ,paramType = "query")
    })
    public MessageResult<Map> isUnbindDeviceList(
            Long personId,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,@RequestAttribute String subject){

        MessageResult<Map> msgResult = new MessageResult<>();

        User user = userService.get(subject);

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        //searchParams.put("companyId",user.getCompanyId());

        Page<DeviceInfo> page = deviceInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        for (DeviceInfo deviceInfo:page) {
            PersonDeviceRelation item = personDeviceRelationService.findByDeviceIdAndPersonId(deviceInfo.getId(),personId);
            if(item == null){
                deviceInfo.setIsBindPerson(false);
            }
            else{
                deviceInfo.setIsBindPerson(true);
            }
            String typeN = dataDictionaryService.findParentId("ddf2ecbe-84a6-4f0d-a84f-2fc6917f5856",deviceInfo.getType());
            deviceInfo.setTypeN(typeN);
        }

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;

    }




}
