package com.jpsoft.smart.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.smart.config.OSSConfig;
import com.jpsoft.smart.modules.common.utils.CheckIdCard;
import com.jpsoft.smart.modules.common.utils.OSSUtil;
import com.jpsoft.smart.modules.common.utils.POIUtils;
import com.jpsoft.smart.modules.common.utils.PojoUtils;
import com.jpsoft.smart.modules.common.dto.Sort;
import com.jpsoft.smart.modules.common.dto.MessageResult;
import com.jpsoft.smart.modules.base.entity.PersonInfo;
import com.jpsoft.smart.modules.base.service.PersonInfoService;
import com.jpsoft.smart.modules.lapi.service.ILapiService;
import com.jpsoft.smart.modules.lapi.vo.LapiMsgResult;
import com.jpsoft.smart.modules.sys.entity.User;
import com.jpsoft.smart.modules.sys.service.UserService;
import com.sun.xml.internal.ws.api.streaming.XMLStreamReaderFactory;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/personInfo")
@Api(description = "personInfo")
public class PersonInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private ILapiService lapiService;
    @Autowired
    private OSSConfig ossConfig;
    @Autowired
    private PersonInfoService personInfoService;
    @Autowired
    private UserService userService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<PersonInfo> create(){
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        PersonInfo personInfo = new PersonInfo();

        msgResult.setData(personInfo);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<PersonInfo> add(@RequestBody PersonInfo personInfo,@RequestAttribute String subject){
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {
            Map<String, Object> searchParams = new HashMap<>();
            searchParams.put("companyId", personInfo.getCompanyId());
            searchParams.put("name", personInfo.getName());
            searchParams.put("phone", personInfo.getPhone());
            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("id_","asc"));
            Page<PersonInfo> page = personInfoService.pageSearch(searchParams, 1, 1,false, sortList);

            if (page.size() > 0) {
                msgResult.setMessage("该人员已存在!");
                msgResult.setResult(false);
                return msgResult;
            } else {
                CheckIdCard cic = null;
                if (StringUtils.isEmpty(personInfo.getIdCard())) {
//                    msgResult.setMessage("身份证不能为空!");
//                    msgResult.setResult(false);
//                    return msgResult;
                } else {
                    personInfo.setIdCard(personInfo.getIdCard().trim().toUpperCase());

                    cic = new CheckIdCard(personInfo.getIdCard());
                    if (!cic.validate()) {
                        msgResult.setMessage("身份证验证失败!");
                        msgResult.setResult(false);
                        return msgResult;
                    }
                }
            }

            personInfo.setDelFlag(false);
            personInfo.setCreateBy(subject);
            personInfo.setCreateTime(new Date());

            int affectCount = personInfoService.insert(personInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(personInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<PersonInfo> edit(@PathVariable("id") Long id){
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.get(id);

            if (personInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(personInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<PersonInfo> update(@RequestBody PersonInfo personInfo,@RequestAttribute String subject){
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {
            personInfo.setUpdateBy(subject);
            personInfo.setUpdateTime(new Date());

            int affectCount = personInfoService.update(personInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(personInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="保存信息")
    @PostMapping("save")
    public MessageResult<PersonInfo> save(@RequestBody PersonInfo personInfo,@RequestAttribute String subject){
        User user = userService.get(subject);
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;
            if(personInfo.getId() != null) {
                personInfo.setCompanyId(user.getCompanyId());
                personInfo.setDelFlag(false);
                personInfo.setCreateBy(subject);
                personInfo.setCreateTime(new Date());

                affectCount = personInfoService.insert(personInfo);
            }else{
                personInfo.setUpdateBy(subject);
                personInfo.setUpdateTime(new Date());

                affectCount = personInfoService.update(personInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(personInfo);
            }else{
                msgResult.setResult(false);
                msgResult.setMessage("数据库保存失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取详情信息")
    @GetMapping("detail/{id}")
    public MessageResult<PersonInfo> detail(@PathVariable("id") Long id){
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.get(id);

            if (personInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(personInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") Long id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.get(id);
            personInfo.setDelFlag(true);
            personInfo.setUpdateBy(subject);
            personInfo.setUpdateTime(new Date());

            int affectCount = personInfoService.update(personInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<Long> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (Long id : idList) {
                PersonInfo personInfo = personInfoService.get(id);
                personInfo.setDelFlag(true);
                personInfo.setUpdateBy(subject);
                personInfo.setUpdateTime(new Date());

                affectCount += personInfoService.update(personInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "name",value = "姓名", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "idCard",value = "身份证", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "phone",value = "手机号", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "companyId",value = "企业id", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "position1",value = "一级位置", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "position2",value = "二级位置", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "position3",value = "三级位置", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "position4",value = "四级位置", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "position5",value = "五级位置", required = false, paramType = "form",dataType = "String")
    })
    public MessageResult<Map> pageList(
            @RequestParam(value="name",defaultValue="") String name,
            @RequestParam(value="idCard",defaultValue="") String idCard,
            @RequestParam(value="phone",defaultValue="") String phone,
            @RequestParam(value="companyId",defaultValue="") String companyId,
            @RequestParam(value="position1",defaultValue="") String position1,
            @RequestParam(value="position2",defaultValue="") String position2,
            @RequestParam(value="position3",defaultValue="") String position3,
            @RequestParam(value="position4",defaultValue="") String position4,
            @RequestParam(value="position5",defaultValue="") String position5,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        User user = userService.get(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("a.id_","asc"));


        if (StringUtils.isNotEmpty(name)) {
            String[] names = name.split(",");
                List nameList = new ArrayList();
                for (String n : names) {
                    nameList.add("%"+ n +"%");
                }
                searchParams.put("nameList", nameList);
        }
        if (StringUtils.isNotEmpty(idCard)) {
            searchParams.put("idCard","%" + idCard + "%");
        }
        if (StringUtils.isNotEmpty(phone)) {
            searchParams.put("phone","%" + phone + "%");
        }

        if (userService.hasRole(subject,"SYSADMIN") && StringUtils.isNotEmpty(companyId)) {
            searchParams.put("companyId",companyId);
        }
        else{
            searchParams.put("companyId",user.getCompanyId());
        }

        if (StringUtils.isNotEmpty(position1)) {
            searchParams.put("position1",position1);
        }
        if (StringUtils.isNotEmpty(position2)) {
            searchParams.put("position2",position2);
        }
        if (StringUtils.isNotEmpty(position3)) {
            searchParams.put("position3",position3);
        }
        if (StringUtils.isNotEmpty(position4)) {
            searchParams.put("position4",position4);
        }
        if (StringUtils.isNotEmpty(position5)) {
            searchParams.put("position5",position5);
        }

        Page<PersonInfo> page = personInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="人脸授权")
    @PostMapping("enabledFace")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "人员编号", required = false, paramType = "form",dataType = "String")
    })
    public MessageResult<PersonInfo> enabledFace(@RequestParam(value="id",defaultValue="") Long id,@RequestAttribute String subject){
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.get(id);

            if(personInfo.getFaceEnabled()){
                if (personInfo==null){
                    throw new Exception("人员信息不存在!");
                }

                //todo 同步终端
                boolean success = true;
                StringBuilder sb = new StringBuilder();
                List<LapiMsgResult> msgResultList = lapiService.deletePerson(id);
                for(int i=0;i<msgResultList.size();i++) {
                    LapiMsgResult lapiMsgResult = msgResultList.get(i);

                    if (lapiMsgResult.isSuccess()){
                        sb.append("【" + personInfo.getName() + "】" + lapiMsgResult.getAliasName() + "解绑人脸信息成功!");
                    }
                    else{
                        sb.append("【" + personInfo.getName() + "】" + lapiMsgResult.getAliasName() + "解绑人脸信息失败!" + lapiMsgResult.getMsg());
                    }

                    if (i!=msgResultList.size()-1){
                        sb.append(",");
                    }
                    else{
                        sb.append("。");
                    }

                    success &= lapiMsgResult.isSuccess();
                }

                personInfo.setFaceBound(false);
                personInfo.setUpdateTime(new Date());
                personInfoService.update(personInfo);
                personInfo.setFaceEnabled(false);
            }else{
                personInfo.setFaceEnabled(true);
            }
            personInfo.setUpdateBy(subject);
            personInfo.setUpdateTime(new Date());

            int affectCount = personInfoService.update(personInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(personInfo);
            }else{
                msgResult.setResult(false);
                msgResult.setMessage("数据库保存失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="人脸批量授权")
    @PostMapping("enabledFaceList")
    public MessageResult<PersonInfo> enabledFaceList(@RequestBody List<String> ids,@RequestAttribute String subject){
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;
            for(String id : ids) {
                PersonInfo personInfo = personInfoService.get(Long.valueOf(id));
                personInfo.setFaceEnabled(true);
                personInfo.setUpdateBy(subject);
                personInfo.setUpdateTime(new Date());
                personInfoService.update(personInfo);

                affectCount++;
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
            }else{
                msgResult.setResult(false);
                msgResult.setMessage("数据库保存失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="刷卡授权")
    @PostMapping("enabledCard")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "人员编号", required = false, paramType = "form",dataType = "String")
    })
    public MessageResult<PersonInfo> enabledCard(@RequestParam(value="id",defaultValue="") Long id,@RequestAttribute String subject){
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.get(id);

            if(personInfo.getCardEnabled()){
                personInfo.setCardEnabled(false);
            }else{
                personInfo.setCardEnabled(true);
            }
            personInfo.setUpdateBy(subject);
            personInfo.setUpdateTime(new Date());

            int affectCount = personInfoService.update(personInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(personInfo);
            }else{
                msgResult.setResult(false);
                msgResult.setMessage("数据库保存失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="手机授权")
    @PostMapping("enabledApp")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "人员编号", required = false, paramType = "form",dataType = "String")
    })
    public MessageResult<PersonInfo> enabledApp(@RequestParam(value="id",defaultValue="") Long id,@RequestAttribute String subject){
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.get(id);

            if(personInfo.getAppEnabled()){
                personInfo.setAppEnabled(false);
            }else{
                personInfo.setAppEnabled(true);
            }
            personInfo.setUpdateBy(subject);
            personInfo.setUpdateTime(new Date());

            int affectCount = personInfoService.update(personInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(personInfo);
            }else{
                msgResult.setResult(false);
                msgResult.setMessage("数据库保存失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="访客授权")
    @PostMapping("enabledGuest")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "人员编号", required = false, paramType = "form",dataType = "String")
    })
    public MessageResult<PersonInfo> enabledGuest(@RequestParam(value="id",defaultValue="") Long id,@RequestAttribute String subject){
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.get(id);

            if(personInfo.getGuestEnabled()){
                personInfo.setGuestEnabled(false);
            }else{
                personInfo.setGuestEnabled(true);
            }
            personInfo.setUpdateBy(subject);
            personInfo.setUpdateTime(new Date());

            int affectCount = personInfoService.update(personInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(personInfo);
            }else{
                msgResult.setResult(false);
                msgResult.setMessage("数据库保存失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="导入人员（非企业账户需要输入企业ID）")
    @PostMapping("importXls")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "companyId",value = "企业ID", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "uploadFile",value = "上传文件", required = true,paramType="form", dataType = "__file")
    })
    public MessageResult<String> importXls(@RequestParam(value="companyId",defaultValue="") String companyId,
                                           MultipartFile uploadFile,
                                           @RequestAttribute String subject){
        User user = userService.get(subject);
        if(StringUtils.isEmpty(companyId)){
            companyId = user.getCompanyId();
        }

        MessageResult<String> msgResult = new MessageResult<>();
        PersonInfo personInfo = new PersonInfo();

        try {
            POIUtils poiUtils = new POIUtils(uploadFile.getInputStream());
            int sheetIndex = 0;
            Sheet sheet1 = poiUtils.getSheetAt(sheetIndex);

            int affectCount = 0;
            int failCount = 0;
            int validateColIndex = 13;

            for(int rowIndex=1 ; rowIndex<=sheet1.getLastRowNum(); rowIndex++){
                try {
                    String name = (String)poiUtils.getCellValue(sheetIndex,rowIndex,1);
                    String idCard = (String)poiUtils.getCellValue(sheetIndex,rowIndex,2);
                    String phone = (String)poiUtils.getCellValue(sheetIndex,rowIndex,3);
                    String position1 = (String)poiUtils.getCellValue(sheetIndex,rowIndex,4);
                    String position2 = (String)poiUtils.getCellValue(sheetIndex,rowIndex,5);
                    String position3 = (String)poiUtils.getCellValue(sheetIndex,rowIndex,6);
                    String position4 = (String)poiUtils.getCellValue(sheetIndex,rowIndex,7);
                    String position5 = (String)poiUtils.getCellValue(sheetIndex,rowIndex,8);
                    String enabledFace = (String)poiUtils.getCellValue(sheetIndex,rowIndex,9);
                    String enabledCard = (String)poiUtils.getCellValue(sheetIndex,rowIndex,10);
                    String enabledApp = (String)poiUtils.getCellValue(sheetIndex,rowIndex,11);
                    String enabledGuest = (String)poiUtils.getCellValue(sheetIndex,rowIndex,12);

                    if(StringUtils.isEmpty(name)){
                        break;
                    }

                    Map<String, Object> searchParams = new HashMap<>();
                    searchParams.put("companyId", companyId);
                    searchParams.put("name", name);
                    searchParams.put("phone", phone);
                    List<Sort> sortList = new ArrayList<>();
                    sortList.add(new Sort("id_","asc"));
                    Page<PersonInfo> page = personInfoService.pageSearch(searchParams, 1, 1,false, sortList);

                    if (page.size() > 0) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("该人员已存在!");
                        failCount++;
                        continue;
                    } else {
                        personInfo.setCompanyId(companyId);
                        personInfo.setName(name);

                        CheckIdCard cic = null;
                        if(StringUtils.isEmpty(idCard)){
//                            sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("身份证不能为空!");
//                            failCount++;
//                            continue;
                        }
                        else{
                            idCard = idCard.trim().toUpperCase();

                            cic = new CheckIdCard(idCard);
                            if (!cic.validate()){
                                sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("身份证验证失败!");
                                failCount++;
                                continue;
                            }
                        }

                        personInfo.setIdCard(idCard);
                        personInfo.setPhone(phone);
                        personInfo.setPosition1(position1);
                        personInfo.setPosition2(position2);
                        personInfo.setPosition3(position3);
                        personInfo.setPosition4(position4);
                        personInfo.setPosition5(position5);
                        if(enabledFace.contains("开")) personInfo.setFaceEnabled(true); else personInfo.setFaceEnabled(false);
                        if(enabledCard.contains("开")) personInfo.setCardEnabled(true); else personInfo.setCardEnabled(false);
                        if(enabledApp.contains("开")) personInfo.setAppEnabled(true); else personInfo.setAppEnabled(false);
                        if(enabledGuest.contains("开")) personInfo.setGuestEnabled(true); else personInfo.setGuestEnabled(false);
                        personInfo.setDelFlag(false);
                        personInfo.setCreateBy(subject);
                        personInfo.setCreateTime(new Date());

                        if (personInfoService.insert(personInfo) > 0) {
                            affectCount++;
                        }
                    }
                }
                catch(Exception innerEx){
                    logger.error(innerEx.getMessage(),innerEx);
                }
            }

            if (failCount>0){
                //有导入失败的记录
                msgResult.setResult(false);
                msgResult.setMessage("数据成功导入" + affectCount + "条,有" + failCount + "条数据未导入成功，错误原因请查看报表。");

                //todo 只保留错误数据的sheet
                Workbook wb = poiUtils.exportErrorXls(sheetIndex,validateColIndex,1 + affectCount + failCount);

                //todo 将wb保存到oss
                ByteArrayOutputStream output = new ByteArrayOutputStream();
                wb.write(output);

                byte[] buffer = output.toByteArray();
                ByteArrayInputStream input = new ByteArrayInputStream(buffer);

                String downloadUrl = OSSUtil.upload(ossConfig,"import","error.xls",input);

                //todo 返回导入失败报表下载链接
                msgResult.setData(downloadUrl);
            }
            else{
                msgResult.setResult(true);
                msgResult.setMessage("数据成功导入" + affectCount + "条");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="导出人员")
    @PostMapping("exportXls")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "name",value = "姓名", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "idCard",value = "身份证", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "phone",value = "手机号", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "companyId",value = "企业id", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "position1",value = "一级位置", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "position2",value = "二级位置", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "position3",value = "三级位置", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "position4",value = "四级位置", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "position5",value = "五级位置", required = false, paramType = "form",dataType = "String")
    })
    public MessageResult<String> exportXls(@RequestParam(value="name",defaultValue="") String name,
                                           @RequestParam(value="idCard",defaultValue="") String idCard,
                                           @RequestParam(value="phone",defaultValue="") String phone,
                                           @RequestParam(value="companyId",defaultValue="") String companyId,
                                           @RequestParam(value="position1",defaultValue="") String position1,
                                           @RequestParam(value="position2",defaultValue="") String position2,
                                           @RequestParam(value="position3",defaultValue="") String position3,
                                           @RequestParam(value="position4",defaultValue="") String position4,
                                           @RequestParam(value="position5",defaultValue="") String position5,
                                           @RequestAttribute String subject){
        MessageResult<String> msgResult = new MessageResult<>();

        try {
            HSSFWorkbook workbook = new HSSFWorkbook();
            HSSFSheet sheet = workbook.createSheet();
            //表头
            HSSFRow rowTitle = sheet.createRow(0);
            HSSFCell cellTitle1 = rowTitle.createCell(0);
            cellTitle1.setCellValue("序号");
            HSSFCell cellTitle2 = rowTitle.createCell(1);
            cellTitle2.setCellValue("姓名");
            HSSFCell cellTitle3 = rowTitle.createCell(2);
            cellTitle3.setCellValue("身份证");
            HSSFCell cellTitle4 = rowTitle.createCell(3);
            cellTitle4.setCellValue("手机号");
            HSSFCell cellTitle5 = rowTitle.createCell(4);
            cellTitle5.setCellValue("位置1");
            HSSFCell cellTitle6 = rowTitle.createCell(5);
            cellTitle6.setCellValue("位置2");
            HSSFCell cellTitle7 = rowTitle.createCell(6);
            cellTitle7.setCellValue("位置3");
            HSSFCell cellTitle8 = rowTitle.createCell(7);
            cellTitle8.setCellValue("位置4");
            HSSFCell cellTitle9 = rowTitle.createCell(8);
            cellTitle9.setCellValue("位置5");
//            HSSFCell cellTitle10 = rowTitle.createCell(9);
//            cellTitle10.setCellValue("人脸开关");
//            HSSFCell cellTitle11 = rowTitle.createCell(10);
//            cellTitle11.setCellValue("刷卡开关");
//            HSSFCell cellTitle12 = rowTitle.createCell(11);
//            cellTitle12.setCellValue("手机开关");
//            HSSFCell cellTitle13 = rowTitle.createCell(12);
//            cellTitle13.setCellValue("访客开关");

            //表内容
            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("a.id_","asc"));
            Map<String,Object> searchParams = new HashMap<>();
//            if (StringUtils.isNotEmpty(name)) {
//                searchParams.put("name","%" + name + "%");
//            }
            if (StringUtils.isNotEmpty(name)) {
                String[] names = name.split(",");
                List nameList = new ArrayList();
                for (String n : names) {
                    nameList.add("%"+ n +"%");
                }
                searchParams.put("nameList", nameList);
            }
            if (StringUtils.isNotEmpty(idCard)) {
                searchParams.put("idCard","%" + idCard + "%");
            }
            if (StringUtils.isNotEmpty(phone)) {
                searchParams.put("phone","%" + phone + "%");
            }
            if (StringUtils.isNotEmpty(companyId)) {
                searchParams.put("companyId",companyId);
            }
            if (StringUtils.isNotEmpty(position1)) {
                searchParams.put("position1",position1);
            }
            if (StringUtils.isNotEmpty(position2)) {
                searchParams.put("position2",position2);
            }
            if (StringUtils.isNotEmpty(position3)) {
                searchParams.put("position3",position3);
            }
            if (StringUtils.isNotEmpty(position4)) {
                searchParams.put("position4",position4);
            }
            if (StringUtils.isNotEmpty(position5)) {
                searchParams.put("position5",position5);
            }

            Page<PersonInfo> page = personInfoService.pageSearch(searchParams,1,10000,false,sortList);

            for(int i=0; i<page.size(); i++){
                PersonInfo personInfo = page.get(i);

                HSSFRow rowContent = sheet.createRow(i + 1);
                HSSFCell cellContent1 = rowContent.createCell(0);
                cellContent1.setCellValue(i+1);
                HSSFCell cellContent2 = rowContent.createCell(1);
                if(personInfo.getName() != null) cellContent2.setCellValue(personInfo.getName());
                HSSFCell cellContent3 = rowContent.createCell(2);
                if(personInfo.getIdCard() != null) cellContent3.setCellValue(personInfo.getIdCard());
                HSSFCell cellContent4 = rowContent.createCell(3);
                if(personInfo.getPhone() != null) cellContent4.setCellValue(personInfo.getPhone());
                HSSFCell cellContent5 = rowContent.createCell(4);
                if(personInfo.getPosition1() != null) cellContent5.setCellValue(personInfo.getPosition1());
                HSSFCell cellContent6 = rowContent.createCell(5);
                if(personInfo.getPosition2() != null) cellContent6.setCellValue(personInfo.getPosition2());
                HSSFCell cellContent7 = rowContent.createCell(6);
                if(personInfo.getPosition3() != null) cellContent7.setCellValue(personInfo.getPosition3());
                HSSFCell cellContent8 = rowContent.createCell(7);
                if(personInfo.getPosition4() != null) cellContent8.setCellValue(personInfo.getPosition4());
                HSSFCell cellContent9 = rowContent.createCell(8);
                if(personInfo.getPosition5() != null) cellContent9.setCellValue(personInfo.getPosition5());
//                HSSFCell cellContent10 = rowContent.createCell(9);
//                if(personInfo.getFaceEnabled() != null && personInfo.getFaceEnabled() == true) cellContent10.setCellValue("开"); else cellContent10.setCellValue("关");
//                HSSFCell cellContent11 = rowContent.createCell(10);
//                if(personInfo.getCardEnabled() != null && personInfo.getCardEnabled() == true) cellContent11.setCellValue("开"); else cellContent11.setCellValue("关");
//                HSSFCell cellContent12 = rowContent.createCell(11);
//                if(personInfo.getAppEnabled() != null && personInfo.getAppEnabled() == true) cellContent12.setCellValue("开"); else cellContent12.setCellValue("关");
//                HSSFCell cellContent13 = rowContent.createCell(12);
//                if(personInfo.getGuestEnabled() != null && personInfo.getGuestEnabled() == true) cellContent13.setCellValue("开"); else cellContent13.setCellValue("关");
            }

            //todo 将wb保存到oss
            ByteArrayOutputStream output = new ByteArrayOutputStream();
            workbook.write(output);

            byte[] buffer = output.toByteArray();
            ByteArrayInputStream input = new ByteArrayInputStream(buffer);

            String downloadUrl = OSSUtil.upload(ossConfig,"import","error.xls",input);

            //todo 返回导入失败报表下载链接

            msgResult.setData(downloadUrl);
            msgResult.setResult(true);
            //将exal输出到哪个文件夹中
//            ByteArrayOutputStream out = new ByteArrayOutputStream();
//            workbook.write(out);
//            workbook.close();
//
//            String fileName = new String("导出数据".getBytes("UTF-8"), "iso-8859-1");
//            response.setContentType("application/x-msdownload");
//            response.setHeader("Content-Disposition", "attachment;fileName=" + fileName + ".xls");
//            response.getOutputStream().write(out.toByteArray());
//            response.getOutputStream().flush();
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="数据同步")
    @PostMapping("dataSync")
    public MessageResult<String> dataSync(@RequestBody List<String> ids){
        MessageResult<String> msgResult = new MessageResult<>();

        try {
            for(String id : ids){
                PersonInfo personInfo = personInfoService.get(Long.valueOf(id));

                if (personInfo==null){
                    throw new Exception("人员信息不存在!");
                }

                //todo 同步终端
                boolean success = true;
                StringBuilder sb = new StringBuilder();
                List<LapiMsgResult> msgResultList = lapiService.addPerson(Long.valueOf(id));
                for(int i=0;i<msgResultList.size();i++) {
                    LapiMsgResult lapiMsgResult = msgResultList.get(i);

                    if (lapiMsgResult.isSuccess()){
                        sb.append("【" + personInfo.getName() + "】" + lapiMsgResult.getAliasName() + "绑定人脸信息成功!");
                    }
                    else{
                        sb.append("【" + personInfo.getName() + "】" + lapiMsgResult.getAliasName() + "绑定人脸信息失败!" + lapiMsgResult.getMsg());
                    }

                    if (i!=msgResultList.size()-1){
                        sb.append(",");
                    }
                    else{
                        sb.append("。");
                    }

                    success &= lapiMsgResult.isSuccess();
                }

                personInfo.setFaceBound(success);
                personInfo.setFaceEnabled(success);
                personInfo.setUpdateTime(new Date());
                personInfoService.update(personInfo);

                msgResult.setResult(true);
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
