package com.jpsoft.smart.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.smart.modules.base.entity.*;
import com.jpsoft.smart.modules.base.service.*;
import com.jpsoft.smart.modules.business.service.RechargeService;
import com.jpsoft.smart.modules.common.dto.MessageResult;
import com.jpsoft.smart.modules.common.dto.Sort;
import com.jpsoft.smart.modules.common.utils.StringUtils;
import com.jpsoft.smart.electricity.server.protocol.MeterReceivePacket;
import com.jpsoft.smart.modules.sys.entity.DataDictionary;
import com.jpsoft.smart.modules.sys.service.DataDictionaryService;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.concurrent.TimeUnit;

@RestController
@RequestMapping("/base/rechargeRecord")
public class RechargeRecordController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private RechargeRecordService rechargeRecordService;
    @Autowired
    private ElectricClientInfoService electricClientInfoService;
    @Autowired
    private DataDictionaryService dataDictionaryService;
    @Autowired
    private RoomInfoService roomInfoService;
    @Autowired
    private TerminalInfoService terminalInfoService;
    @Autowired
    private ElectricMeterInfoService electricMeterInfoService;
    @Autowired
    private RechargeService rechargeService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<RechargeRecord> create(@RequestAttribute String subject){
        MessageResult<RechargeRecord> msgResult = new MessageResult<>();

        RechargeRecord rechargeRecord = new RechargeRecord();

        msgResult.setData(rechargeRecord);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<RechargeRecord> add(@RequestBody RechargeRecord rechargeRecord,@RequestAttribute String subject){
        MessageResult<RechargeRecord> msgResult = new MessageResult<>();

        try {
            rechargeRecord.setId(UUID.randomUUID().toString());
            rechargeRecord.setDelFlag(false);
            rechargeRecord.setCreateBy(subject);
            rechargeRecord.setCreateTime(new Date());
            
            int affectCount = rechargeRecordService.insert(rechargeRecord);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(rechargeRecord);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<RechargeRecord> edit(@PathVariable("id") String id){
        MessageResult<RechargeRecord> msgResult = new MessageResult<>();

        try {
            RechargeRecord rechargeRecord = rechargeRecordService.get(id);

            if (rechargeRecord != null) {
                msgResult.setResult(true);
                msgResult.setData(rechargeRecord);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<RechargeRecord> update(@RequestBody RechargeRecord rechargeRecord,@RequestAttribute String subject){
        MessageResult<RechargeRecord> msgResult = new MessageResult<>();

        try {
            rechargeRecord.setUpdateBy(subject);
            rechargeRecord.setUpdateTime(new Date());
            
            int affectCount = rechargeRecordService.update(rechargeRecord);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(rechargeRecord);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            RechargeRecord rechargeRecord = rechargeRecordService.get(id);
            rechargeRecord.setDelFlag(true);
            rechargeRecord.setUpdateBy(subject);
            rechargeRecord.setUpdateTime(new Date());

            int affectCount = rechargeRecordService.update(rechargeRecord);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                RechargeRecord rechargeRecord = rechargeRecordService.get(id);
                rechargeRecord.setDelFlag(true);
                rechargeRecord.setUpdateBy(subject);
                rechargeRecord.setUpdateTime(new Date());

                affectCount += rechargeRecordService.update(rechargeRecord);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(name = "buyType", defaultValue = "") String buyType,
            @RequestParam(name = "paymentStatus", defaultValue = "") String paymentStatus,
            @RequestParam(name = "areaId", defaultValue = "") String areaId,
            @RequestParam(name = "buildingId", defaultValue = "") String buildingId,
            @RequestParam(name = "roomsId", defaultValue = "") String roomsId,
            @RequestParam(name="serialNumber",defaultValue = "") String serialNumber,
            @RequestParam(name="pageIndex",defaultValue = "1") int pageIndex,
            @RequestParam(name="pageSize",defaultValue = "10") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        if (StringUtils.isNotEmpty(serialNumber)) {
            searchParams.put("serialNumber","%" + serialNumber + "%");
        }

        if (StringUtils.isNotEmpty(buyType)) {
            searchParams.put("buyType", buyType);
        }
        if (StringUtils.isNotEmpty(paymentStatus)) {
            searchParams.put("paymentStatus", paymentStatus);
        }

        //先判断是否选择房间
        if (StringUtils.isNotEmpty(roomsId)) {
            //选择了房间
            List<String> roomsIds = new ArrayList();
            roomsIds.add(roomsId);
            searchParams.put("roomsIds", roomsIds);
        }else if (StringUtils.isNotEmpty(buildingId)) {
            //没有选择房间,选择楼栋
            List<String> buildingIds = new ArrayList();
            buildingIds.add(buildingId);
            List<String> roomsIds = roomInfoService.findIdByTypeAndParentIds("3",buildingIds);
            if(roomsIds.size() < 1){
                roomsIds.add("");
            }
            searchParams.put("roomsIds", roomsIds);
        }else if (StringUtils.isNotEmpty(areaId)) {
            //没有选择房间和楼栋,选择的园区
            List<String> areaIds = new ArrayList();
            areaIds.add(areaId);
            List<String> buildingIds = roomInfoService.findIdByTypeAndParentIds("2",areaIds);
            List<String> roomsIds = roomInfoService.findIdByTypeAndParentIds("3",buildingIds);
            if(roomsIds.size() < 1){
                roomsIds.add("");
            }
            searchParams.put("roomsIds", roomsIds);
        }

        Page<RechargeRecord> page = rechargeRecordService.pageSearch(searchParams,pageIndex,pageSize,sortList);
        List<RechargeRecord> resultRecordList = new ArrayList<>();
        List<RechargeRecord> recordList = page.getResult();

        for(RechargeRecord rechargeRecord :recordList){
            RoomInfo room = roomInfoService.get(rechargeRecord.getRoomId());
            RoomInfo building = new RoomInfo();
            RoomInfo area = new RoomInfo();
            if(room != null){
                building = roomInfoService.get(room.getParentId());
                if(building != null){
                    area = roomInfoService.get(building.getParentId());
                }
            }

            rechargeRecord.setRoom(room);
            rechargeRecord.setBuilding(building);
            rechargeRecord.setArea(area);
            resultRecordList.add(rechargeRecord);
        }

        Map<String,Object> pageMap = new HashMap<>();
        pageMap.put("recordsTotal",page.getTotal());
        pageMap.put("recordsFiltered",page.getTotal());
        pageMap.put("totalPage",page.getPages());
        pageMap.put("pageNumber",page.getPageNum());
        pageMap.put("pageSize",page.getPageSize());
        pageMap.put("data", resultRecordList);

        msgResult.setResult(true);
        msgResult.setData(pageMap);

        return msgResult;
    }

    @ApiOperation(value="保存电量")
    @PostMapping("saveRecord")
    public MessageResult<RechargeRecord> saveRecord(@RequestBody RechargeRecord rechargeRecord,@RequestAttribute String subject) throws  Exception {
        MessageResult<RechargeRecord> msgResult = new MessageResult<>();
        boolean isOk = true;
        try {

            ElectricClientInfo clientInfo = electricClientInfoService.get(rechargeRecord.getClientId());

            if(clientInfo==null){
                throw new Exception("该房间没有绑定电表!");
            }

            ElectricMeterInfo meterInfo = electricMeterInfoService.get(clientInfo.getMeterId());
            TerminalInfo terminalInfo = terminalInfoService.get(meterInfo.getTerminalId());

            if (terminalInfo==null){
                throw new Exception("该房间绑定电表没有设置对应终端!");
            }

            //新增电量等信息补充到电表中
            //BigDecimal buyAmount = rechargeRecord.getBuyAmount();//购买金额
            Integer buyElectricity = rechargeRecord.getBuyElectricity();//购买电量

            //充电
            MeterReceivePacket meterReceivePacket = rechargeService.recharge(clientInfo.getRoomId(), buyElectricity, 10, TimeUnit.SECONDS);
            //MeterReceivePacket meterReceivePacket = new MeterReceivePacket();
            //meterReceivePacket.setValidateResult(true);
            //MeterReceivePacket meterReceivePacket = null;
            if (meterReceivePacket == null) {
                rechargeRecord.setErrorLog("线下充电失败-返回null");
                isOk = false;
            } else {
                if (meterReceivePacket.isValidateResult()) {
                    rechargeRecord.setDumpEnergy(meterReceivePacket.getRemain()); //当前剩余电量

                    Integer electricityTotal = clientInfo.getElectricityTotal();//原总电量

                    if (electricityTotal == null) {
                        electricityTotal = 0;
                    }

                    Integer newElectricityTotal = electricityTotal + buyElectricity;//总购电量

                    clientInfo.setElectricityRemaining(meterReceivePacket.getRemain());
                    clientInfo.setRechargeTimes(meterReceivePacket.getRechargeTimes());

                    clientInfo.setActive(meterReceivePacket.getState()!=1);

                    clientInfo.setElectricityTotal(newElectricityTotal);

                    clientInfo.setUpdateBy(subject);
                    clientInfo.setUpdateTime(new Date());
                    electricClientInfoService.update(clientInfo);
                } else {
                    isOk = false;
                    rechargeRecord.setErrorLog("线下充电失败-返回状态为false");
                }
            }

            String serialNumber = StringUtils.getOutTradeNo();
            rechargeRecord.setSerialNumber(serialNumber);
            rechargeRecord.setId(UUID.randomUUID().toString());
            rechargeRecord.setDelFlag(false);
            rechargeRecord.setCreateBy(subject);
            rechargeRecord.setCreateTime(new Date());
            rechargeRecord.setBuyType("cash");

            if(isOk){
                rechargeRecord.setPaymentStatus("20");
                rechargeRecord.setChargingStatus("20");
            }else{
                rechargeRecord.setPaymentStatus("30");
                rechargeRecord.setChargingStatus("30");
            }
            rechargeRecord.setRoomId(clientInfo.getRoomId());
            rechargeRecord.setPaymentTime(new Date());
            rechargeRecord.setCustomerNo(clientInfo.getCustomerNo());

            int affectCount = rechargeRecordService.insert(rechargeRecord);
            if (affectCount > 0) {
                if(isOk){
                    msgResult.setResult(true);
                    msgResult.setData(rechargeRecord);
                }else{
                    msgResult.setResult(false);
                    msgResult.setMessage(rechargeRecord.getErrorLog());
                }
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="线下充值前查询房间信息")
    @GetMapping("queryRoomMessage/{clientId}")
    public MessageResult<Map<String,Object>> queryRoomMessage(@PathVariable("clientId") String clientId,@RequestAttribute String subject) throws Exception {
        MessageResult<Map<String,Object>> msgResult = new MessageResult<>();

        ElectricClientInfo clientInfo = electricClientInfoService.getRoomMessage(clientId);        //充值类型

        if (clientInfo==null){
            throw new Exception("该房间绑定电表!");
        }

        RoomInfo roomInfo = roomInfoService.get(clientInfo.getRoomId());
        DataDictionary dataDictionary = dataDictionaryService.get(roomInfo.getUseType());
        if(dataDictionary == null){
            throw new Exception("该房间没有绑定用电类型!");
        }
        Map<String,Object> resultMaps = new HashMap();
        resultMaps.put("roomId",clientInfo.getRoomId());
        resultMaps.put("useTypeN",dataDictionary.getName());//中文翻译
        resultMaps.put("useTypeV",dataDictionary.getValue());//值
        msgResult.setData(resultMaps);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value="创建交易信息")
    @GetMapping("createSerial")
    public MessageResult<String> createSerial(@RequestAttribute String subject){
        MessageResult<String> msgResult = new MessageResult<>();

        String serialNumber = StringUtils.getOutTradeNo();

        RechargeRecord rechargeRecord = new RechargeRecord();
        rechargeRecord.setId(UUID.randomUUID().toString());
        rechargeRecord.setDelFlag(false);
        rechargeRecord.setCreateBy(subject);
        rechargeRecord.setCreateTime(new Date());
        rechargeRecord.setSerialNumber(serialNumber);
        rechargeRecord.setPaymentStatus("10");//10为未支付 20为支付成功
        rechargeRecord.setChargingStatus("10");//10为未充电 20为充电成功

        int affectCount = rechargeRecordService.insert(rechargeRecord);
        if (affectCount > 0) {
            msgResult.setResult(true);
            msgResult.setData(serialNumber);
        } else {
            msgResult.setResult(false);
            msgResult.setMessage("创建失败");
        }
        return msgResult;
    }


}
