package com.jpsoft.smart.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.smart.modules.common.utils.PojoUtils;
import com.jpsoft.smart.modules.common.dto.Sort;
import com.jpsoft.smart.modules.common.dto.MessageResult;
import com.jpsoft.smart.modules.base.entity.WarningPusher;
import com.jpsoft.smart.modules.base.service.WarningPusherService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/warningPusher")
@Api(description = "warningPusher")
public class WarningPusherController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private WarningPusherService warningPusherService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<WarningPusher> create(){
        MessageResult<WarningPusher> msgResult = new MessageResult<>();

        WarningPusher warningPusher = new WarningPusher();

        msgResult.setData(warningPusher);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<WarningPusher> add(@RequestBody WarningPusher warningPusher,@RequestAttribute String subject){
        MessageResult<WarningPusher> msgResult = new MessageResult<>();

        try {
            warningPusher.setId(UUID.randomUUID().toString());
            warningPusher.setDelFlag(false);
            warningPusher.setCreateBy(subject);
            warningPusher.setCreateTime(new Date());
            
            int affectCount = warningPusherService.insert(warningPusher);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(warningPusher);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<WarningPusher> edit(@PathVariable("id") String id){
        MessageResult<WarningPusher> msgResult = new MessageResult<>();

        try {
            WarningPusher warningPusher = warningPusherService.get(id);

            if (warningPusher != null) {
                msgResult.setResult(true);
                msgResult.setData(warningPusher);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<WarningPusher> update(@RequestBody WarningPusher warningPusher,@RequestAttribute String subject){
        MessageResult<WarningPusher> msgResult = new MessageResult<>();

        try {
            warningPusher.setUpdateBy(subject);
            warningPusher.setUpdateTime(new Date());
            
            int affectCount = warningPusherService.update(warningPusher);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(warningPusher);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            WarningPusher warningPusher = warningPusherService.get(id);
            warningPusher.setDelFlag(true);
            warningPusher.setUpdateBy(subject);
            warningPusher.setUpdateTime(new Date());

            int affectCount = warningPusherService.update(warningPusher);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                WarningPusher warningPusher = warningPusherService.get(id);
                warningPusher.setDelFlag(true);
                warningPusher.setUpdateBy(subject);
                warningPusher.setUpdateTime(new Date());

                affectCount += warningPusherService.update(warningPusher);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<WarningPusher> page = warningPusherService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
