package com.jpsoft.smart.modules.business.controller;

import com.jpsoft.smart.electricity.server.AttributeKeys;
import com.jpsoft.smart.electricity.server.ChannelWrapper;
import com.jpsoft.smart.electricity.server.protocol.MeterReceivePacket;
import com.jpsoft.smart.electricity.server.protocol.MeterSendPacket;
import io.netty.channel.Channel;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Slf4j
@RequestMapping("/demo")
@RestController
public class DemoController {
    @Autowired
    private Map<String, ChannelWrapper> activeChannelMap;

    @RequestMapping(value="hello",method= RequestMethod.GET)
    public String hello(){
        return "hello world!";
    }

    @ApiOperation(value="获取已连接客户端")
    @RequestMapping(value="channelList",method= RequestMethod.GET)
    public List<String> channelList(){
        List<String> list = new ArrayList<>();

        for(String key : activeChannelMap.keySet()){
            Channel channel = activeChannelMap.get(key).getChannel();
;
            list.add(key + "," + channel.remoteAddress().toString() + "," + channel.attr(AttributeKeys.TERMINAL_NO));
        }

        return list;
    }

    @ApiOperation(value="查询电表")
    @ApiImplicitParams({
        @ApiImplicitParam(name="terminalNo",value="终端号",dataType="string",paramType = "query")
    })
    @RequestMapping(value="query",method= RequestMethod.GET)
    public MeterReceivePacket query(String terminalNo,
                                    @RequestParam(defaultValue = "1") String machineNo,
                                    @RequestParam(defaultValue = "3000") String unitNo,
                                    @RequestParam(defaultValue = "8") int houseNo) {
        MeterReceivePacket mrp = new MeterReceivePacket();

        try {
            MeterSendPacket msp = new MeterSendPacket();
            msp.setMachineNo(Integer.valueOf(machineNo,16));
            msp.setUnitNo(Integer.valueOf(unitNo,16));
            msp.setHouseNo(houseNo);
            msp.setCommand(MeterSendPacket.READ);
            msp.setRequestTime(new Date());

            activeChannelMap.get(terminalNo).getChannel().writeAndFlush(msp);

            //从队列中读取
            String key = machineNo + "-" + unitNo + "-" + houseNo;
            mrp = activeChannelMap.get(terminalNo).getReceivePacketQueue(key).poll(10, TimeUnit.SECONDS);
        } catch (InterruptedException e) {
            log.error(e.getMessage(),e);
        }

        return mrp;
    }

    @ApiOperation(value="充电")
    @ApiImplicitParams({
            @ApiImplicitParam(name="channelId",value="通道号",dataType="string",paramType = "query"),
            @ApiImplicitParam(name="num",value="充电度数",dataType="int",paramType = "query")
    })
    @RequestMapping(value="recharge",method= RequestMethod.GET)
    public MeterReceivePacket recharge(String terminalNo,
                                    @RequestParam(defaultValue = "1") String machineNo,
                                    @RequestParam(defaultValue = "3000") String unitNo,
                                    @RequestParam(defaultValue = "8") int houseNo,
                                    @RequestParam(defaultValue = "1") int num) {
        MeterReceivePacket mrp = new MeterReceivePacket();

        try {
            MeterReceivePacket queryPacket = query(terminalNo,machineNo,unitNo,houseNo);

            MeterSendPacket msp = new MeterSendPacket();
            msp.setMachineNo(Integer.valueOf(machineNo,16));
            msp.setUnitNo(Integer.valueOf(unitNo,16));
            msp.setHouseNo(houseNo);
            msp.setCommand(MeterSendPacket.RECHARGE);
            msp.setRechargeTimes(queryPacket.getRechargeTimes() + 1);
            msp.setRechargeNum(num);
            msp.setRequestTime(new Date());

            activeChannelMap.get(terminalNo).getChannel().writeAndFlush(msp);

            //从队列中读取
            String key = machineNo + "-" + unitNo + "-" + houseNo;
            mrp = activeChannelMap.get(terminalNo).getReceivePacketQueue(key).poll(10, TimeUnit.SECONDS);
        } catch (InterruptedException e) {
            log.error(e.getMessage(),e);
        }

        return mrp;
    }

    @ApiOperation(value="通电")
    @ApiImplicitParams({
            @ApiImplicitParam(name="channelId",value="通道号",dataType="string",paramType = "query")
    })
    @RequestMapping(value="on",method= RequestMethod.GET)
    public MeterReceivePacket on(String terminalNo,
                                    @RequestParam(defaultValue = "1") String machineNo,
                                    @RequestParam(defaultValue = "3000") String unitNo,
                                    @RequestParam(defaultValue = "8") int houseNo) {
        MeterReceivePacket mrp = new MeterReceivePacket();

        try {
            MeterSendPacket msp = new MeterSendPacket();
            msp.setMachineNo(Integer.valueOf(machineNo,16));
            msp.setUnitNo(Integer.valueOf(unitNo,16));
            msp.setHouseNo(houseNo);
            msp.setCommand(MeterSendPacket.ON);
            msp.setRequestTime(new Date());

            activeChannelMap.get(terminalNo).getChannel().writeAndFlush(msp);

            //从队列中读取
            String key = machineNo + "-" + unitNo + "-" + houseNo;
            mrp = activeChannelMap.get(terminalNo).getReceivePacketQueue(key).poll(10, TimeUnit.SECONDS);
        } catch (InterruptedException e) {
            log.error(e.getMessage(),e);
        }

        return mrp;
    }

    @ApiOperation(value="断电")
    @ApiImplicitParams({
            @ApiImplicitParam(name="channelId",value="通道号",dataType="string",paramType = "query")
    })
    @RequestMapping(value="off",method= RequestMethod.GET)
    public MeterReceivePacket off(String terminalNo,
                                    @RequestParam(defaultValue = "1") String machineNo,
                                    @RequestParam(defaultValue = "3000") String unitNo,
                                    @RequestParam(defaultValue = "8") int houseNo) {
        MeterReceivePacket mrp = new MeterReceivePacket();

        try {
            MeterSendPacket msp = new MeterSendPacket();
            msp.setMachineNo(Integer.valueOf(machineNo,16));
            msp.setUnitNo(Integer.valueOf(unitNo,16));
            msp.setHouseNo(houseNo);
            msp.setCommand(MeterSendPacket.OFF);
            msp.setRequestTime(new Date());

            activeChannelMap.get(terminalNo).getChannel().writeAndFlush(msp);

            //从队列中读取
            String key = machineNo + "-" + unitNo + "-" + houseNo;
            mrp = activeChannelMap.get(terminalNo).getReceivePacketQueue(key).poll(10, TimeUnit.SECONDS);
        } catch (InterruptedException e) {
            log.error(e.getMessage(),e);
        }

        return mrp;
    }
}
