package com.jpsoft.smart.modules.business.controller;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.Page;
import com.jpsoft.smart.config.RedisConfig;
import com.jpsoft.smart.modules.base.entity.*;
import com.jpsoft.smart.modules.base.service.*;
import com.jpsoft.smart.modules.business.service.RechargeService;
import com.jpsoft.smart.modules.common.dto.MessageResult;
import com.jpsoft.smart.modules.common.dto.Sort;
import com.jpsoft.smart.modules.common.utils.StringUtils;
import com.jpsoft.smart.electricity.server.protocol.MeterReceivePacket;
import com.jpsoft.smart.electricity.server.protocol.MeterSendPacket;
import com.jpsoft.smart.modules.pay.properties.WxJpsoftProperties;
import com.jpsoft.smart.modules.sys.entity.DataDictionary;
import com.jpsoft.smart.modules.sys.service.DataDictionaryService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.web.bind.annotation.*;

import javax.net.ssl.HttpsURLConnection;
import java.io.InputStream;
import java.math.BigDecimal;
import java.net.URL;
import java.text.DecimalFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;

@Slf4j
@RequestMapping("/mobileApi")
@RestController
public class MobileApiController {
    @Autowired
    private RoomInfoService roomInfoService;

    @Autowired
    private RechargeService rechargeService;

    @Autowired
    private ElectricClientInfoService electricClientInfoService;

    @Autowired
    private DataDictionaryService dataDictionaryService;

    @Autowired
    private RechargeRecordService rechargeRecordService;

    @Autowired
    ValueOperations<String, Object> valueOperations;

    @Autowired
    private WxJpsoftProperties wxJpsoftProperties;

    /**
     * 查询房间
     * @param type 1:园区,2:楼栋,3:房间
     * @param parentId 上级编号
     * @return
     */
    @GetMapping("queryRoomList")
    public MessageResult<List> queryRoomList(String type,String parentId){
        MessageResult<List> msgResult = new MessageResult<>();

        try {
            List<RoomInfo> areaList = roomInfoService.findByTypeAndParentId(type,parentId);
            msgResult.setData(areaList);
            msgResult.setCode(200);
            msgResult.setResult(true);
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setCode(500);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="查询电量")
    @ApiImplicitParams({
            @ApiImplicitParam(name="roomId",value="房间号",dataType="string",paramType = "query")
    })
    @GetMapping(value="queryPower")
    public MessageResult<MeterReceivePacket> queryPower(String roomId) {
        MessageResult<MeterReceivePacket> msgResult = new MessageResult<>();

        try {
            MeterReceivePacket mrp = rechargeService.sendCommand(roomId, MeterSendPacket.READ,10,TimeUnit.SECONDS);

            msgResult.setData(mrp);
            msgResult.setCode(200);
            msgResult.setResult(true);
        } catch (Exception e) {
            msgResult.setResult(false);
            msgResult.setCode(500);
            msgResult.setMessage(e.getMessage());

            log.error(e.getMessage(),e);
        }

        return msgResult;
    }


    @ApiOperation(value="打开或关闭电源")
    @PostMapping("powerSwitch")
    public MessageResult<MeterReceivePacket> powerSwitch(String roomId, int cmd) {
        MessageResult<MeterReceivePacket> msgResult = new MessageResult<>();

        try {
            MeterReceivePacket mrp = null;

            if(cmd==1) {
                mrp = rechargeService.sendCommand(roomId, MeterSendPacket.ON,5, TimeUnit.SECONDS);
            }
            else {
                mrp = rechargeService.sendCommand(roomId, MeterSendPacket.OFF,5,TimeUnit.SECONDS);
            }

            if(mrp!=null && mrp.isValidateResult()){
                ElectricClientInfo clientInfo = electricClientInfoService.findOneByRoomId(roomId);

                clientInfo.setActive(mrp.getState()!=1);

                electricClientInfoService.update(clientInfo);

                msgResult.setResult(true);
                msgResult.setData(mrp);
            }
            else{
                msgResult.setResult(false);
                msgResult.setMessage("命令超时!");
            }

        } catch (Exception e) {
            log.error(e.getMessage(),e);

            msgResult.setMessage(e.getMessage());
            msgResult.setResult(false);
        }

        return msgResult;
    }

    @GetMapping(value="queryRoomDetail")
    public MessageResult<Map> queryRoomDetail(String roomId) {
        MessageResult<Map> msgResult = new MessageResult<>();

        try {
            Map<String,Object> map = new HashMap<>();

            RoomInfo room = roomInfoService.get(roomId);
            RoomInfo building = roomInfoService.get(room.getParentId());
            RoomInfo area = roomInfoService.get(building.getParentId());

            map.put("room",room);
            map.put("building",building);
            map.put("area",area);

            //查询用电类型及价格
            DataDictionary dd = dataDictionaryService.get(room.getUseType());

            if(dd==null){
                throw new Exception("该房间用电类型未设置!");
            }

            //价格
            map.put("price",dd.getValue());

            msgResult.setData(map);
            msgResult.setResult(true);
        } catch (Exception e) {
            msgResult.setResult(false);
            msgResult.setCode(500);
            msgResult.setMessage(e.getMessage());

            log.error(e.getMessage(),e);
        }

        return msgResult;
    }

    @GetMapping(value="queryRecordDetail")
    @ApiImplicitParams({
            @ApiImplicitParam(name="serialNum",value="交易流水号",dataType="string",paramType = "query")
    })
    public MessageResult<RechargeRecord> queryRecordDetail(String serialNum) {
        MessageResult<RechargeRecord> msgResult = new MessageResult<>();

        try {
            Map<String,Object> map = new HashMap<>();
            RechargeRecord rechargeRecord = rechargeRecordService.getBySerialNum(serialNum);

            msgResult.setData(rechargeRecord);
            msgResult.setResult(true);
        } catch (Exception e) {
            msgResult.setResult(false);
            msgResult.setCode(500);
            msgResult.setMessage(e.getMessage());

            log.error(e.getMessage(),e);
        }

        return msgResult;
    }

    private RechargeRecord createRechargeRecord(String roomId, String buyType, Integer num, BigDecimal amount) throws Exception{
        String serialNumber = StringUtils.getOutTradeNo();

        RoomInfo room = roomInfoService.get(roomId);

        //查询用电类型及价格
        DataDictionary dd = dataDictionaryService.get(room.getUseType());

        BigDecimal price = new BigDecimal(dd.getValue());

        if(num<=0){
            throw new Exception("充电度数只能为正整数!");
        }

        BigDecimal totalAmount = price.multiply(new BigDecimal(num));

        DecimalFormat df = new DecimalFormat("#.##");

        //支付金额>=单价*数量
        if(totalAmount.compareTo(amount)<0){
            throw  new Exception("支付金额应为:" + df.format(totalAmount) + "!");
        }

        RechargeRecord record = new RechargeRecord();
        record.setId(UUID.randomUUID().toString());
        record.setDelFlag(false);
        record.setCreateTime(new Date());
        record.setSerialNumber(serialNumber);
        record.setProductTheme("电费充值(测试)");
        record.setPaymentStatus("10");//10为未支付 20为支付成功
        record.setChargingStatus("10");//10为未充电 20为充电成功
        record.setRoomId(roomId);
        record.setBuyElectricity(num);
        record.setBuyAmount(totalAmount);
        record.setBuyType(buyType); //weipay alipay cash

        int affectCount = rechargeRecordService.insert(record);

        if (affectCount>0){
            return record;
        }
        else{
            return null;
        }
    }

    private String getCommonAccessToken() throws Exception{
        String accessToken = null;

        accessToken = (String)valueOperations.get(RedisConfig.WX_COMMON_ACCESS_TOKEN);

        if(StringUtils.isEmpty(accessToken)){
            URL realUrl = new URL(String.format("https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=%s&secret=%s",
                    wxJpsoftProperties.getAppId(), wxJpsoftProperties.getAppSecret()));

            HttpsURLConnection conn = (HttpsURLConnection) realUrl.openConnection();

            conn.connect();

            InputStream input = conn.getInputStream();

            ObjectMapper objectMapper = new ObjectMapper();

            Map resultMap = objectMapper.readValue(input, Map.class);

            accessToken = (String)resultMap.get("access_token");
            Integer expiresIn = Integer.valueOf(resultMap.get("expires_in").toString()); //默认是7200秒过期

            valueOperations.set(RedisConfig.WX_COMMON_ACCESS_TOKEN,accessToken,expiresIn,TimeUnit.SECONDS);
        }

        return accessToken;
    }

    private String getJSApiTicket() throws Exception{
        String ticket = null;

        ticket = (String)valueOperations.get(RedisConfig.JS_API_TICKET);

        if(StringUtils.isEmpty(ticket)){
            String accessToken = getCommonAccessToken();

            URL realUrl = new URL(String.format("https://api.weixin.qq.com/cgi-bin/ticket/getticket?type=jsapi&access_token=%s",
                    accessToken));

            HttpsURLConnection conn = (HttpsURLConnection) realUrl.openConnection();

            conn.connect();

            InputStream input = conn.getInputStream();

            ObjectMapper objectMapper = new ObjectMapper();

            Map resultMap = objectMapper.readValue(input, Map.class);

            Integer errcode = (Integer)resultMap.get("errcode");

            if(errcode.equals(0)){
                ticket = (String)resultMap.get("ticket");
                Integer expiresIn = Integer.valueOf(resultMap.get("expires_in").toString()); //默认是7200秒过期

                valueOperations.set(RedisConfig.JS_API_TICKET,ticket,expiresIn,TimeUnit.SECONDS);
            }
        }

        return ticket;
    }

    public Map getAuthAccessToken(String code) throws Exception{
        String tokenUrl = String.format("https://api.weixin.qq.com/sns/oauth2/access_token?appid=%s&secret=%s&code=%s&grant_type=authorization_code",
                wxJpsoftProperties.getAppId(), wxJpsoftProperties.getAppSecret(),code);
        String key = "AUTH_" + code;

        Map tokenMap = (Map)valueOperations.get(key);

        if(tokenMap==null){
            URL realUrl = new URL(tokenUrl);

            HttpsURLConnection conn = (HttpsURLConnection) realUrl.openConnection();

            conn.connect();

            InputStream input = conn.getInputStream();

            ObjectMapper objectMapper = new ObjectMapper();

            tokenMap = objectMapper.readValue(input, Map.class);

            valueOperations.set(key,tokenMap,5,TimeUnit.MINUTES);
        }

        return tokenMap;
    }

    @PostMapping(value="prepareWXPay")
    public MessageResult<Map> prepareWXPay(String roomId, String buyType, Integer num, BigDecimal amount){
        MessageResult<Map> msgResult = new MessageResult<>();

        try {
            Map<String,Object> dataMap = new HashMap<>();

            dataMap.put("appId", wxJpsoftProperties.getAppId());
            RechargeRecord record = createRechargeRecord(roomId,buyType,num,amount);

            if(record!=null){
                dataMap.put("recordId",record.getId());
            }

            msgResult.setResult(true);

            msgResult.setData(dataMap);
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setMessage(ex.getMessage());
            msgResult.setResult(false);
        }

        return msgResult;
    }

    @GetMapping(value="getWXConfigParam")
    public MessageResult<Map> getWXConfigParam(String code){
        MessageResult<Map> msgResult = new MessageResult<>();

        try {
            Map<String,Object> dataMap = new HashMap<>();

            dataMap.put("appId", wxJpsoftProperties.getAppId());
            dataMap.put("ticket", getJSApiTicket());

            Map tokenMap = getAuthAccessToken(code);

            String accessToken = (String)tokenMap.get("access_token");
            String openId = (String)tokenMap.get("openid");

            dataMap.put("accessToken",accessToken);
            dataMap.put("openId",openId);

            msgResult.setResult(true);

            msgResult.setData(dataMap);
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setMessage(ex.getMessage());
            msgResult.setResult(false);
        }

        return msgResult;
    }

    @PostMapping(value="prepareAliPay")
    public MessageResult<Map> prepareAliPay(String roomId, String buyType, Integer num, BigDecimal amount){
        MessageResult<Map> msgResult = new MessageResult<>();

        try {
            Map<String,Object> dataMap = new HashMap<>();

            RechargeRecord record = createRechargeRecord(roomId,buyType,num,amount);

            if(record!=null){
                dataMap.put("recordId",record.getId());
            }

            msgResult.setResult(true);

            msgResult.setData(dataMap);
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setMessage(ex.getMessage());
            msgResult.setResult(false);
        }

        return msgResult;
    }

    @GetMapping(value="queryRechargeRecord")
    public MessageResult<RechargeRecord> queryRechargeRecord(String recordId){
        MessageResult<RechargeRecord> msgResult = new MessageResult<>();

        try {
            if (StringUtils.isEmpty(recordId)){
                throw new Exception("记录编号不能为空!");
            }

            RechargeRecord record = rechargeRecordService.get(recordId);

            msgResult.setResult(true);
            msgResult.setData(record);
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setMessage(ex.getMessage());
            msgResult.setResult(false);
        }

        return msgResult;
    }

    @GetMapping(value="rechargeList")
    public MessageResult<Map> rechargeList(String roomId,
                                           @RequestParam(defaultValue = "1") Integer pageNum,
                                           @RequestParam(defaultValue = "10") Integer pageSize){
        if(pageSize>20){
            pageSize = 20;
        }

        MessageResult<Map> msgResult = new MessageResult<>();

        try {
            if (StringUtils.isEmpty(roomId)){
                throw new Exception("房间编号不能为空!");
            }

            Map<String,Object> dataMap = new HashMap<>();

            Map<String,Object> searchParams = new HashMap<>();

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time","desc"));

            searchParams.put("roomId",roomId);
            searchParams.put("paymentStatus","20");

            Page<RechargeRecord> page = rechargeRecordService.pageSearch(searchParams,pageNum,pageSize,sortList);

            dataMap.put("rows",page);
            dataMap.put("pageNum",page.getPageNum());
            dataMap.put("pageSize",page.getPageSize());
            dataMap.put("totalRows",page.getTotal());

            msgResult.setResult(true);

            msgResult.setData(dataMap);
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setMessage(ex.getMessage());
            msgResult.setResult(false);
        }

        return msgResult;
    }
}
