package com.jpsoft.smart.modules.business.service.impl;

import com.jpsoft.smart.modules.base.entity.ElectricClientInfo;
import com.jpsoft.smart.modules.base.entity.ElectricMeterInfo;
import com.jpsoft.smart.modules.base.entity.TerminalInfo;
import com.jpsoft.smart.modules.base.service.ElectricClientInfoService;
import com.jpsoft.smart.modules.base.service.ElectricMeterInfoService;
import com.jpsoft.smart.modules.base.service.RoomInfoService;
import com.jpsoft.smart.modules.base.service.TerminalInfoService;
import com.jpsoft.smart.modules.business.service.RechargeService;
import com.jpsoft.smart.electricity.server.ChannelWrapper;
import com.jpsoft.smart.electricity.server.protocol.MeterReceivePacket;
import com.jpsoft.smart.electricity.server.protocol.MeterSendPacket;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import java.util.Date;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Component(value="rechargeService")
public class RechargeServiceImpl implements RechargeService {
    @Autowired
    private RoomInfoService roomInfoService;

    @Autowired
    private ElectricClientInfoService electricClientInfoService;

    @Autowired
    private ElectricMeterInfoService electricMeterInfoService;

    @Autowired
    private TerminalInfoService terminalInfoService;

    @Autowired
    private Map<String, ChannelWrapper> activeChannelMap;

    @Override
    public MeterReceivePacket sendCommand(String roomId,int command,int timeout,TimeUnit unit) throws  Exception {
        MeterReceivePacket mrp = null;

        ElectricClientInfo clientInfo = electricClientInfoService.findOneByRoomId(roomId);

        if(clientInfo==null){
            throw new Exception("该房间没有绑定电表!");
        }

        ElectricMeterInfo meterInfo = electricMeterInfoService.get(clientInfo.getMeterId());
        TerminalInfo terminalInfo = terminalInfoService.get(meterInfo.getTerminalId());

        if (terminalInfo==null){
            throw new Exception("该房间绑定电表没有设置对应终端!");
        }

        String terminalNo = terminalInfo.getName();
        String machineNo = meterInfo.getMeterNo();
        String unitNo = meterInfo.getUnitNo();
        String houseNo = clientInfo.getCustomerNo();

        if(activeChannelMap.containsKey(terminalNo)) {
            MeterSendPacket msp = new MeterSendPacket();
            msp.setMachineNo(Integer.valueOf(machineNo, 16));
            msp.setUnitNo(Integer.valueOf(unitNo, 16));
            msp.setHouseNo(Integer.valueOf(houseNo));
            msp.setCommand(command);
            msp.setRequestTime(new Date());

            activeChannelMap.get(terminalNo).getChannel().writeAndFlush(msp);

            //从队列中读取
            String key = machineNo + "-" + unitNo + "-" + houseNo;
            mrp = activeChannelMap.get(terminalNo).getReceivePacketQueue(key).poll(timeout, unit);
        }
        else{
            throw new Exception(String.format("该房间对应电表%s-%s未上线!",machineNo,unitNo));
        }

        return mrp;
    }

    @Override
    public MeterReceivePacket recharge(String roomId, int num, int timeout,TimeUnit unit) throws Exception {
        MeterReceivePacket mrp = null;
        MeterReceivePacket queryPacket = sendCommand(roomId,MeterSendPacket.READ,timeout,unit);

        if(queryPacket!=null){
            MeterSendPacket msp = new MeterSendPacket();
            msp.setMachineNo(queryPacket.getMachineNo());
            msp.setUnitNo(queryPacket.getUnitNo());
            msp.setHouseNo(queryPacket.getHouseNo());
            msp.setCommand(MeterSendPacket.RECHARGE);
            msp.setRechargeTimes(queryPacket.getRechargeTimes() + 1);
            msp.setRechargeNum(num);
            msp.setRequestTime(new Date());

            activeChannelMap.get(queryPacket.getTerminalNo()).getChannel().writeAndFlush(msp);

            //从队列中读取

            String key = Integer.toHexString(queryPacket.getMachineNo()) + "-" +
                    Integer.toHexString(queryPacket.getUnitNo()) + "-" + queryPacket.getHouseNo();

            mrp = activeChannelMap.get(queryPacket.getTerminalNo()).getReceivePacketQueue(key).poll(timeout, unit);
        }

        return mrp;
    }
}
