package com.jpsoft.smart.modules.business.task;

import com.github.pagehelper.Page;
import com.jpsoft.smart.modules.base.entity.ElectricClientInfo;
import com.jpsoft.smart.modules.base.entity.RechargeRecord;
import com.jpsoft.smart.modules.base.service.ElectricClientInfoService;
import com.jpsoft.smart.modules.base.service.RechargeRecordService;
import com.jpsoft.smart.modules.business.service.RechargeService;
import com.jpsoft.smart.modules.common.dto.Sort;
import com.jpsoft.smart.electricity.server.protocol.MeterReceivePacket;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.concurrent.TimeUnit;

@Slf4j
@Component
public class RechargeTask {
    @Autowired
    private RechargeService rechargeService;

    @Autowired
    private RechargeRecordService rechargeRecordService;

    @Autowired
    private ElectricClientInfoService electricClientInfoService;

    @Scheduled(cron = "0/30 * * * * *")
    public void recharge(){
        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","asc"));

        //查询已支付但未充电记录
        searchParams.put("paymentStatus","20");
        searchParams.put("chargingStatus","10");

        Page<RechargeRecord> page = rechargeRecordService.pageSearch(searchParams,1,20,false, sortList);

        for (RechargeRecord record : page) {
            try {
                MeterReceivePacket packet =  rechargeService.recharge(record.getRoomId(), record.getBuyElectricity(), 5, TimeUnit.SECONDS);

                if(packet!=null && packet.isValidateResult()){
                    record.setChargingStatus("20");
                    record.setDumpEnergy(packet.getRemain()); //当前剩余电量

                    ElectricClientInfo clientInfo = electricClientInfoService.findOneByRoomId(record.getRoomId());

                    if (clientInfo!=null){
                        //记录剩余电量
                        clientInfo.setElectricityRemaining(packet.getRemain());

                        //记录充电次数
                        clientInfo.setRechargeTimes(packet.getRechargeTimes());

                        clientInfo.setActive(packet.getState()!=1);

                        clientInfo.setUpdateTime(new Date());

                        electricClientInfoService.update(clientInfo);
                    }
                }
            }
            catch(Exception ex){
                record.setErrorLog(ex.getMessage());
                log.error(ex.getMessage(),ex);
            }

            record.setUpdateTime(new Date());

            rechargeRecordService.update(record);
        }
    }
}
