package com.jpsoft.smart.modules.pay.alipay;


import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayClient;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.internal.util.AlipaySignature;
import com.alipay.api.request.AlipayTradePrecreateRequest;
import com.alipay.api.response.AlipayTradePrecreateResponse;
import com.ijpay.alipay.AliPayApi;
import com.jpsoft.smart.modules.base.entity.ElectricClientInfo;
import com.jpsoft.smart.modules.base.entity.RechargeRecord;
import com.jpsoft.smart.modules.base.service.ElectricClientInfoService;
import com.jpsoft.smart.modules.base.service.RechargeRecordService;
import com.jpsoft.smart.modules.business.service.RechargeService;
import com.jpsoft.smart.modules.common.dto.MessageResult;
import com.jpsoft.smart.electricity.server.protocol.MeterReceivePacket;
import com.jpsoft.smart.modules.pay.properties.AliPayProperties;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author 墨鱼_mo
 * @date 2019-11-26 9:48
 */

@Slf4j
@RequestMapping("/aliPay")
@RestController
public class AlipayController {

    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private AliPayProperties aliPayProperties;

    @Autowired
    private RechargeRecordService rechargeRecordService;

    @Autowired
    private RechargeService rechargeService;

    @Autowired
    private ElectricClientInfoService electricClientInfoService;

    @ApiOperation(value = "支付宝扫码支付")
    @GetMapping("/tradePrecreatePay")
    public MessageResult tradePrecreatePay(String recordId) {

        MessageResult result = new MessageResult();

        try {
            AlipayClient alipayClient = new DefaultAlipayClient(aliPayProperties.getServiceUrl(), aliPayProperties.getAppId(), aliPayProperties.getPrivateKey(), "json", aliPayProperties.getInputCharset(), aliPayProperties.getZfbPublicKey(), aliPayProperties.getSignType());
            AlipayTradePrecreateRequest request = new AlipayTradePrecreateRequest();
            request.setNotifyUrl(aliPayProperties.getNotifyUrl());

            RechargeRecord rechargeRecord = rechargeRecordService.get(recordId);

            if (rechargeRecord == null) {
                throw new Exception("订单不存在");
            }
            if (rechargeRecord.getBuyAmount().compareTo(BigDecimal.ZERO) != 1) {
                throw new Exception("金额有误");
            }
            if (rechargeRecord.getPaymentStatus().equals("20")) {
                throw new Exception("订单已支付");
            }


            Map<String, Object> maps = new HashMap<>();
            maps.put("out_trade_no", rechargeRecord.getSerialNumber());
            maps.put("total_amount", String.valueOf(rechargeRecord.getBuyAmount()));
            maps.put("subject", rechargeRecord.getProductTheme());
            maps.put("timeout_express", "5m");

           /* JSONObject extendParams = new JSONObject();

            extendParams.put("sys_service_provider_id", aliPayProperties.getMchId());
            maps.put("extend_params", extendParams);
            request.putOtherTextParam("app_auth_token", aliPayProperties.getAppAuthToken());*/
            //把订单信息转换为json对象的字符串
            String postdata = JSONObject.toJSONString(maps);
            request.setBizContent(postdata);
            AlipayTradePrecreateResponse response = alipayClient.execute(request);
            String body = response.getBody();
            JSONObject jsonObject = JSONObject.parseObject(body);
            String alipayTradePrecreateResponse = jsonObject.getString("alipay_trade_precreate_response");
            JSONObject jsonResponse = JSONObject.parseObject(alipayTradePrecreateResponse);

            if (!jsonResponse.getString("code").equals("10000")) {
                throw new Exception(jsonResponse.getString("sub_msg"));
            }

            String qr_code = jsonObject.getJSONObject("alipay_trade_precreate_response").getString("qr_code");
            result.setResult(true);

            log.warn("qr_code=" + qr_code);
            result.setData(qr_code);

        } catch (Exception e) {
            e.printStackTrace();
            logger.error(e.getMessage(), e);
            result.setResult(false);
            result.setMessage(e.getMessage());
        }
        return result;

    }

    @RequestMapping(value="/payNotify")
    public String payNotify(HttpServletRequest request) {
        try {
            // 获取支付宝POST过来反馈信息
            Map<String, String> params = AliPayApi.toMap(request);

            for (Map.Entry<String, String> entry : params.entrySet()) {
                System.out.println(entry.getKey() + " = " + entry.getValue());
            }

            boolean verifyResult = AlipaySignature.rsaCheckV1(params, aliPayProperties.getZfbPublicKey(), "UTF-8", "RSA2");

            if (verifyResult) {

                RechargeRecord rechargeRecord = rechargeRecordService.getBySerialNum(params.get("out_trade_no"));
                if (rechargeRecord == null) {
                    throw new Exception("支付已完成，但未找到此订单号的流水");
                }
                if (!rechargeRecord.getBuyAmount().toString().equals(params.get("buyer_pay_amount"))) {
                    throw new Exception("支付金额与系统金额不一致");
                }
                rechargeRecord.setUpdateTime(new Date());
                rechargeRecord.setUpdateBy(params.get("buyer_logon_id"));
                rechargeRecord.setBuyType("alipay");
                rechargeRecord.setPaymentStatus("20");
                rechargeRecord.setTransactionNumber(params.get("trade_no"));
                rechargeRecord.setPaymentTime(DateUtil.parse(params.get("gmt_payment")));

                try {
                    MeterReceivePacket mp = rechargeService.recharge(rechargeRecord.getRoomId(), rechargeRecord.getBuyElectricity(), 3, TimeUnit.SECONDS);
                    if (mp == null){
                        rechargeRecord.setErrorLog("相应超时");
                    }else {
                        if (!mp.isValidateResult()){
                            rechargeRecord.setErrorLog("校验失败");
                        }else {
                            rechargeRecord.setChargingStatus("20");
                            rechargeRecord.setDumpEnergy(mp.getRemain()); //当前剩余电量

                            ElectricClientInfo clientInfo = electricClientInfoService.findOneByRoomId(rechargeRecord.getRoomId());

                            if (clientInfo!=null){
                                //记录剩余电量
                                clientInfo.setElectricityRemaining(mp.getRemain());

                                //记录充电次数
                                clientInfo.setRechargeTimes(mp.getRechargeTimes());
                                clientInfo.setActive(mp.getState()!=1);

                                clientInfo.setUpdateTime(new Date());

                                electricClientInfoService.update(clientInfo);
                            }
                        }
                    }
                } catch (Exception e) {
                    rechargeRecord.setErrorLog(e.getMessage());
                    e.printStackTrace();
                    logger.error(e.getMessage(), e);
                }
                rechargeRecordService.update(rechargeRecord);


                return "success";
            } else {
                logger.error("notify_url 验证失败");
                return "failure";
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error(e.getMessage(), e);
            return "failure";
        }
    }

}
